import re
from .dataset import provinces as plate_provinces, categories as plate_categories
from .dataset import TYPE_CAR, TYPE_MOTORCYCLE
from .exceptions import InvalidPlateLength


def normalize(plate: str):
    non_digit_regex = r'\D'
    char = ''.join(re.findall(non_digit_regex, plate)) or None
    numbers = re.sub(non_digit_regex, '', plate)
    return {'char': char, 'numbers': numbers}


def _car_info(char: str, numbers: str):
    province_code = int(numbers[5:7])
    _type = TYPE_CAR
    template = numbers[0:2] + str(char) + numbers[2:5] + 'ایران' + str(province_code)

    province = plate_provinces[_type].get(province_code)
    category = plate_categories.get(char)

    return {
        'type': _type,
        'template': template,
        'province': province,
        'category': category
    }


def _motorcycle_info(numbers: str):
    province_code = int(numbers[0:3])
    _type = TYPE_MOTORCYCLE
    template = str(province_code) + '-' + numbers[3:]

    province = plate_provinces[_type].get(province_code)

    return {
        'type': _type,
        'template': template,
        'province': province,
        'category': None
    }


def get_info(plate: str):
    normal_plate = normalize(plate)

    if len(normal_plate['numbers']) == 7:
        return _car_info(normal_plate['char'], normal_plate['numbers'])

    if len(normal_plate['numbers']) == 8:
        return _motorcycle_info(normal_plate['numbers'])

    raise InvalidPlateLength(plate)


def is_valid(plate: str):
    normal_plate = normalize(plate)
    numbers = normal_plate['numbers']

    for num in numbers[:-1]:
        if num == '0':
            return False

    if len(numbers) < 7 or len(numbers) > 8:
        return False

    info = get_info(plate)
    if info.get('type') == TYPE_CAR and info.get('category') is None:
        return False

    if info.get('province') is None:
        return False

    return True



from persian_tools.bank import card_number
from persian_tools.bank.exceptions import InvalidCardNumber
import pytest


def test_validate():
    assert card_number.validate('6037701689095443')
    assert card_number.validate('6219861034529007')
    assert card_number.validate('6219861034529008') is False
    assert card_number.validate('621986103452900') is False
    assert card_number.validate('0') is False


def test_bank_name():
    assert card_number.bank_data('6037701689095443').get('persian_name') == 'بانک کشاورزی'
    assert card_number.bank_data('6219861034529007').get('persian_name') == 'بانک سامان'
    assert card_number.bank_data('6219861034529007').get('persian_name') == 'بانک سامان'

    assert card_number.bank_data('1319861034529007') is None

    with pytest.raises(InvalidCardNumber, match='.*is an invalid card number'):
        card_number.bank_data('621986103452900')
        card_number.bank_data('9999991034529007')


mock_string = '''شماره کارتم رو برات نوشتم:
    6219-8610-3452-9007
    اینم یه شماره کارت دیگه ای که دارم
    5022291070873466
    ۵۰۲۲۲۹۱۰۸۱۸۷۳۴۶۶
    ۵۰۲۲-۲۹۱۰-۷۰۸۷-۳۴۶۶'''


def test_extract_card_numbers_1():
    """Should find and extract 4 Card Numbers"""

    expected = [
        {'pure': '6219861034529007', 'base': '6219-8610-3452-9007', 'index': 1},
        {'pure': '5022291070873466', 'base': '5022291070873466', 'index': 2},
        {'pure': '5022291081873466', 'base': '۵۰۲۲۲۹۱۰۸۱۸۷۳۴۶۶', 'index': 3},
        {'pure': '5022291070873466', 'base': '۵۰۲۲-۲۹۱۰-۷۰۸۷-۳۴۶۶', 'index': 4},
    ]
    actual = card_number.extract_card_numbers(mock_string, check_validation=False)

    assert len(actual) == 4
    assert len(actual) == len(expected)
    assert all([a == b for a, b in zip(actual, expected)])


def test_extract_card_numbers_2():
    """Should find and format the Card-Number into Text that includes Persian & English digits"""
    mock_string_2 = '''شماره کارتم رو برات نوشتم: ۵۰۲۲-2910-7۰۸۷-۳۴۶۶'''

    expected = [
        {'pure': '5022291070873466', 'base': '۵۰۲۲-2910-7۰۸۷-۳۴۶۶', 'index': 1},
    ]
    actual = card_number.extract_card_numbers(mock_string_2, check_validation=False)

    assert len(actual) == 1
    assert len(actual) == len(expected)
    assert all([a == b for a, b in zip(actual, expected)])


def test_extract_card_numbers_3():
    """Should validate extract card-numbers"""

    expected = [
        {'pure': '6219861034529007', 'base': '6219-8610-3452-9007', 'index': 1, 'is_valid': True},
        {'pure': '5022291070873466', 'base': '5022291070873466', 'index': 2, 'is_valid': True},
        {'pure': '5022291081873466', 'base': '۵۰۲۲۲۹۱۰۸۱۸۷۳۴۶۶', 'index': 3, 'is_valid': False},
        {'pure': '5022291070873466', 'base': '۵۰۲۲-۲۹۱۰-۷۰۸۷-۳۴۶۶', 'index': 4, 'is_valid': True},
    ]
    actual = card_number.extract_card_numbers(mock_string, check_validation=True, filter_valid_card_numbers=False)

    assert len(actual) == 4
    assert len(actual) == len(expected)
    assert all([a == b for a, b in zip(actual, expected)])


def test_extract_card_numbers_4():
    """Should return only valid card-numbers"""

    expected = [
        {'pure': '6219861034529007', 'base': '6219-8610-3452-9007', 'index': 1, 'is_valid': True},
        {'pure': '5022291070873466', 'base': '5022291070873466', 'index': 2, 'is_valid': True},
        {'pure': '5022291070873466', 'base': '۵۰۲۲-۲۹۱۰-۷۰۸۷-۳۴۶۶', 'index': 4, 'is_valid': True},
    ]
    actual = card_number.extract_card_numbers(mock_string, check_validation=True, filter_valid_card_numbers=True)

    assert len(actual) == 3
    assert len(actual) == len(expected)
    assert all([a == b for a, b in zip(actual, expected)])

    actual_without_attr = card_number.extract_card_numbers(mock_string)

    assert len(actual) == len(actual_without_attr)
    assert all([a == b for a, b in zip(actual, actual_without_attr)])


def test_extract_card_numbers_5():
    """Should detect Banks number for valid card-numbers"""

    expected = [
        {'pure': '6219861034529007', 'base': '6219-8610-3452-9007', 'index': 1, 'is_valid': True,
         'bank_data': {
             'nickname': 'saman',
             'name': 'Saman Bank',
             'persian_name': 'بانک سامان',
             'card_prefix': ['621986'],
             'sheba_code': ['056'],
         }},
        {'pure': '5022291070873466', 'base': '5022291070873466', 'index': 2, 'is_valid': True,
         'bank_data': {
             'nickname': 'pasargad',
             'name': 'Pasargad Bank',
             'persian_name': 'بانک پاسارگاد',
             'card_prefix': ['502229', '639347'],
             'sheba_code': ['057'],
         }},
        {'pure': '5022291070873466', 'base': '۵۰۲۲-۲۹۱۰-۷۰۸۷-۳۴۶۶', 'index': 4, 'is_valid': True,
         'bank_data': {
             'nickname': 'pasargad',
             'name': 'Pasargad Bank',
             'persian_name': 'بانک پاسارگاد',
             'card_prefix': ['502229', '639347'],
             'sheba_code': ['057'],
         }},
    ]

    actual = card_number.extract_card_numbers(mock_string, check_validation=True, filter_valid_card_numbers=True,
                                              detect_bank_name=True)

    assert len(actual) == 3
    assert len(actual) == len(expected)
    assert all([a == b for a, b in zip(actual, expected)])



from persian_tools.bank import sheba
from persian_tools.bank.exceptions import InvalidShebaNumber
import pytest


def test_validate():
    assert sheba.validate('IR820540102680020817909002')

    assert sheba.validate('IR01234567890123456789') is False
    assert sheba.validate('IR012345678901234567890123456789') is False
    assert sheba.validate('IR01234567890123456789') is False
    assert sheba.validate('IR012345678901234567890123') is False
    assert sheba.validate('IR012345678901234567890123') is False
    assert sheba.validate('01234567890123456789012345') is False


def test_bank_data():
    actual = sheba.bank_data('IR820540102680020817909002')
    expected = {'nickname': 'parsian', 'name': 'Parsian Bank', 'persian_name': 'بانک پارسیان',
                'card_prefix': ['622106', '627884'], 'sheba_code': ['054'], 'account_number': '020817909002',
                'formatted_account_number': '002-00817909-002'}

    assert all(v == actual[k] for k, v in expected.items())
    assert len(expected) == len(actual)

    actual = sheba.bank_data('IR550570022080013447370101')
    expected = {'nickname': 'pasargad', 'name': 'Pasargad Bank', 'persian_name': 'بانک پاسارگاد',
                'card_prefix': ['502229', '639347'], 'sheba_code': ['057'], 'account_number': '220800134473701',
                'formatted_account_number': '220-800-13447370-1'}

    assert all(v == actual[k] for k, v in expected.items())
    assert len(expected) == len(actual)

    with pytest.raises(InvalidShebaNumber, match='.*is an invalid sheba number'):
        sheba.bank_data('IR012345678901234567890123')
        sheba.bank_data('IR012345678A01234567890123')



from persian_tools import digits


def test_convert_to_fa():
    assert digits.convert_to_fa('123٤٥٦') == '۱۲۳۴۵۶'
    assert digits.convert_to_fa(123456) == '۱۲۳۴۵۶'


def test_convert_to_ar():
    assert digits.convert_to_ar('123۴۵۶') == '١٢٣٤٥٦'
    assert digits.convert_to_ar(123.456) == '١٢٣.٤٥٦'


def test_convert_to_en():
    assert digits.convert_to_en('۱۲۳٤٥٦') == '123456'


def test_conversion_function():
    assert digits._conversion('123', 'de') is None


def test_convert_to_word():
    assert digits.convert_to_word(500443) == 'پانصد هزار و چهارصد و چهل و سه'
    assert len(digits.convert_to_word(500)) == 5
    assert digits.convert_to_word(30000000000) == 'سی میلیارد'
    assert digits.convert_to_word(30000000000000) == 'سی بیلیون'
    assert digits.convert_to_word(30000000000000000) == 'سی بیلیارد'
    assert digits.convert_to_word(30000000000000000000) is None
    assert digits.convert_to_word(0) == 'صفر'

    assert digits.convert_to_word(500443, ordinal=True) == 'پانصد هزار و چهارصد و چهل و سوم'
    assert digits.convert_to_word(-30, ordinal=True) == 'منفی سی اُم'
    assert digits.convert_to_word(33, ordinal=True) == 'سی و سوم'
    assert digits.convert_to_word(45, ordinal=True) == 'چهل و پنجم'



from typing import Union
from .exceptions import InvalidShebaNumber
from .banks_code import data as banks_code


def _iso7064Mod97_10(iban: str) -> int:
    reminder = iban

    while len(reminder) > 2:
        block = reminder[:9]
        reminder = str(int(block) % 97) + reminder[len(block):]

    return int(reminder) % 97


def validate(sheba: str) -> bool:
    if len(sheba) != 26:
        return False

    if sheba[:2] != 'IR' or not sheba[2:].isdigit():
        return False

    d1 = ord(sheba[0]) - 65 + 10
    d2 = ord(sheba[1]) - 65 + 10

    new_code = sheba[4:] + str(d1) + str(d2) + sheba[2:4]
    reminder = _iso7064Mod97_10(new_code)

    return reminder == 1


def bank_data(sheba: str) -> Union[dict, None]:
    if not validate(sheba):
        raise InvalidShebaNumber(sheba)

    sheba_code = sheba[4:7]
    bank = [bank for bank in banks_code if sheba_code in bank.get('sheba_code', [])]

    if len(bank) < 1:
        return None

    data = bank[0].copy()
    if callable(data.get('account_number_calculator')):
        account_data = data['account_number_calculator'](sheba)
        data['account_number'] = account_data['normal']
        data['formatted_account_number'] = account_data['formatted']
        del data['account_number_calculator']
    return data



import re
from typing import Union
from .operators import data as operators_data
from .exceptions import InvalidPhoneNumber

MOBILE_REGEX = '(?:[+|0{2}]?98)?(?:0)?(\\d{3})+(\\d{3})+(\\d{4})'


def _get_operator_data(phone_number: str) -> Union[dict, bool]:
    found_number = re.findall(MOBILE_REGEX, phone_number)
    if found_number:
        prefix = found_number[0][0]
        return operators_data.get(prefix, False)

    return False


def validate(phone_number: str) -> bool:
    return _get_operator_data(phone_number) is not False


def operator_data(phone_number: str) -> bool:
    res = _get_operator_data(phone_number)

    if res is False:
        raise InvalidPhoneNumber(phone_number)

    return res



from persian_tools import ordinal_suffix


def test_add_ordinal_suffix():
    assert ordinal_suffix.add('چهل و سه') == 'چهل و سوم'
    assert ordinal_suffix.add('چهل و پنج') == 'چهل و پنجم'
    assert ordinal_suffix.add('سی') == 'سی اُم'


def test_remove_ordinal_suffix():
    assert ordinal_suffix.remove('چهل و سوم') == 'چهل و سه'
    assert ordinal_suffix.remove('چهل و پنجم') == 'چهل و پنج'
    assert ordinal_suffix.remove('سی اُم') == 'سی'



TYPE_CAR = 'Car'
TYPE_MOTORCYCLE = 'Motorcycle'

provinces = {
    TYPE_CAR: {
        10: 'تهران',
        11: 'تهران',
        12: 'خراسان رضوی',
        13: 'اصفحان',
        14: 'خوزستان',
        15: 'آذربایجان شرقی',
        16: 'قم',
        17: 'آذربایجان غربی',
        18: 'همدان',
        19: 'کرمانشاه',
        20: 'تهران',
        21: 'تهران - البرز',
        22: 'تهران',
        23: 'اصفحان',
        24: 'خوزستان',
        25: 'آذربایجان شرقی',
        26: 'خراسان شمالی',
        27: 'آذربایجان غربی',
        28: 'همدان',
        29: 'کرمانشاه',
        30: 'تهران - البرز',
        31: 'لرستان',
        32: 'خراسان شمالی - خراسان جنوبی - خراسان رضوی',
        33: 'تهران',
        34: 'خوزستان',
        35: 'آذربایجان شرقی',
        36: 'خراسان رضوی',
        37: 'آذربایجان غربی',
        38: 'تهران - البرز',
        40: 'تهران',
        41: 'لرستان',
        42: 'خراسان رضوی',
        43: 'اصفحان',
        44: 'تهران',
        45: 'یزد',
        46: 'گیلان',
        47: 'مرکزی',
        48: 'بوشهر',
        49: 'کهگیلویه و بویراحمد',
        51: 'کردستان',
        52: 'خراسان جنوبی',
        53: 'اصفحان',
        55: 'تهران',
        56: 'گیلان',
        57: 'مرکزی',
        58: 'بوشهر',
        59: 'گلستان',
        61: 'کردستان',
        62: 'مازندران',
        63: 'فارس',
        64: 'یزد',
        65: 'کرمان',
        66: 'تهران',
        67: 'اصفحان',
        68: 'البرز',
        69: 'گلستان',
        71: 'چهارمحال و بختیاری',
        72: 'مازندران',
        73: 'فارس',
        74: 'خراسان رضوی',
        75: 'کرمان',
        76: 'گیلان',
        77: 'تهران',
        78: 'تهران - البرز',
        79: 'قزوین',
        81: 'چهارمحال و بختیاری',
        82: 'مازندران',
        83: 'فارس',
        84: 'هرمزگان',
        85: 'سیستان و بلوچستان',
        86: 'سمنان',
        87: 'زنجان',
        88: 'تهران',
        89: 'قزوین',
        91: 'اردبیل',
        92: 'مازندران',
        93: 'فارس',
        94: 'هرمزگان',
        95: 'سیستان و بلوچستان',
        96: 'سمنان',
        97: 'زنجان',
        98: 'ایلام',
        99: 'تهران',
    },
    TYPE_MOTORCYCLE: {
        111: 'مرکز تهران',
        112: 'مرکز تهران',
        113: 'مرکز تهران',
        114: 'مرکز تهران',
        115: 'مرکز تهران',
        116: 'مرکز تهران',
        117: 'مرکز تهران',
        118: 'مرکز تهران',
        119: 'مرکز تهران',
        120: 'مرکز تهران',
        121: 'مرکز تهران',
        122: 'مرکز تهران',
        123: 'مرکز تهران',
        124: 'مرکز تهران',
        125: 'مرکز تهران',
        126: 'مرکز تهران',
        127: 'مرکز تهران',
        128: 'مرکز تهران',
        129: 'مرکز تهران',
        130: 'مرکز تهران',
        131: 'مرکز تهران',
        132: 'مرکز تهران',
        133: 'مرکز تهران',
        134: 'مرکز تهران',
        135: 'مرکز تهران',
        136: 'مرکز تهران',
        137: 'مرکز تهران',
        138: 'مرکز تهران',
        319: 'البرز و شهرستان های تهران',
        321: 'البرز و شهرستان های تهران',
        322: 'البرز و شهرستان های تهران',
        323: 'البرز و شهرستان های تهران',
        324: 'البرز و شهرستان های تهران',
        371: 'آذربایجان غربی',
        372: 'آذربایجان غربی',
        373: 'آذربایجان غربی',
        374: 'آذربایجان غربی',
        375: 'آذربایجان غربی',
        376: 'آذربایجان غربی',
        377: 'آذربایجان غربی',
        391: 'آذربایجان شرقی',
        392: 'آذربایجان شرقی',
        393: 'آذربایجان شرقی',
        394: 'آذربایجان شرقی',
        395: 'آذربایجان شرقی',
        396: 'آذربایجان شرقی',
        397: 'آذربایجان شرقی',
        442: 'اردبیل',
        443: 'اردبیل',
        461: 'کردستان',
        462: 'کردستان',
        479: 'زنجان',
        481: 'زنجان',
        482: 'زنجان',
        498: 'همدان',
        499: 'همدان',
        511: 'همدان',
        514: 'کرمانشاه',
        515: 'کرمانشاه',
        516: 'کرمانشاه',
        517: 'کرمانشاه',
        523: 'قزوین',
        524: 'قزوین',
        525: 'قزوین',
        531: 'مرکزی',
        532: 'مرکزی',
        533: 'مرکزی',
        534: 'مرکزی',
        535: 'مرکزی',
        538: 'لرستان',
        539: 'لرستان',
        540: 'لرستان',
        541: 'لرستان',
        542: 'لرستان',
        543: 'لرستان',
        547: 'ایلام',
        555: 'چهارمحال و بختیاری',
        561: 'خوزستان',
        562: 'خوزستان',
        563: 'خوزستان',
        564: 'خوزستان',
        565: 'خوزستان',
        566: 'خوزستان',
        567: 'خوزستان',
        568: 'خوزستان',
        569: 'خوزستان',
        571: 'کهگیلویه و بویراحمد',
        578: 'گیلان',
        579: 'گیلان',
        581: 'گیلان',
        582: 'گیلان',
        586: 'مازندران',
        587: 'مازندران',
        588: 'مازندران',
        589: 'مازندران',
        596: 'گلستان',
        597: 'گلستان',
        611: 'قم',
        612: 'قم',
        613: 'قم',
        614: 'قم',
        637: 'یزد',
        638: 'یزد',
        639: 'یزد',
        641: 'یزد',
        642: 'یزد',
        643: 'یزد',
        687: 'فارس',
        688: 'فارس',
        689: 'فارس',
        691: 'فارس',
        692: 'فارس',
        693: 'فارس',
        694: 'فارس',
        695: 'فارس',
        696: 'فارس',
        697: 'فارس',
        751: 'سمنان',
        752: 'سمنان',
        753: 'سمنان',
        761: 'خراسان رضوی',
        762: 'خراسان رضوی',
        763: 'خراسان رضوی',
        764: 'خراسان رضوی',
        765: 'خراسان رضوی',
        766: 'خراسان رضوی',
        767: 'خراسان رضوی',
        768: 'خراسان رضوی',
        769: 'خراسان رضوی',
        771: 'خراسان رضوی',
        772: 'خراسان رضوی',
        773: 'خراسان رضوی',
        774: 'خراسان رضوی',
        775: 'خراسان رضوی',
        776: 'خراسان رضوی',
        777: 'خراسان رضوی',
        779: 'خراسان شمالی',
        781: 'خراسان شمالی',
        791: 'خراسان جنوبی',
        792: 'خراسان جنوبی',
        812: 'کرمان',
        813: 'کرمان',
        814: 'کرمان',
        815: 'کرمان',
        816: 'کرمان',
        817: 'کرمان',
        819: 'سیستان و بلوچستان',
        821: 'سیستان و بلوچستان',
        822: 'سیستان و بلوچستان',
        823: 'سیستان و بلوچستان',
        827: 'بوشهر',
        828: 'بوشهر',
        829: 'بوشهر',
        831: 'بوشهر',
        835: 'هرمزگان',
        836: 'هرمزگان',
        837: 'هرمزگان',
        838: 'هرمزگان',
        839: 'هرمزگان',
    }
}

categories = {
    'D': 'دیپلمات',
    'S': 'سفارتخانه ها ',
    'ف': 'ستاد کل نیرو های مسلح',
    'ز': 'وزارت دفاع و پشتیبانی',
    'ش': 'ارتش',
    'تشریفات': 'تشریفات',
    'الف': 'دولتی',
    'پ': 'پلیس',
    'ث': 'سپاه',
    'ک': 'کشاورزی',
    'ع': 'حمل و نقل عمومی',
    'گ': 'گذر موقت',
    'ژ': 'معلولان و جانبازان',
    'ب': 'شخصی',
    'ج': 'شخصی',
    'د': 'شخصی',
    'س': 'شخصی',
    'ص': 'شخصی',
    'ط': 'شخصی',
    'ق': 'شخصی',
    'ل': 'شخصی',
    'م': 'شخصی',
    'ن': 'شخصی',
    'و': 'شخصی',
    'ه': 'شخصی',
    'ی': 'شخصی',
}



from typing import Union
import random
from .places_code import data as places_code
from .exceptions import InvalidNationalId
from ..digits import convert_to_en


def validate(national_id: str) -> bool:
    code = national_id.zfill(10)

    if not code.isdigit() or len(code) != 10:
        return False

    if int(code[3:-1]) == 0:
        return False

    _sum = 0
    for i in range(9):
        _sum += int(code[i]) * (10 - i)

    remaining = _sum % 11
    check_number = int(code[-1])

    if remaining < 2:
        return check_number == remaining
    return check_number == 11 - remaining


def generate_random() -> str:
    random_number = random.randint(10 ** 6, 10 ** 9 - 1)
    random_number = str(random_number).zfill(9)

    _sum = 0
    for i in range(9):
        _sum += int(random_number[i]) * (10 - i)

    remaining = _sum % 11
    if remaining < 2:
        last_number = remaining
    else:
        last_number = 11 - remaining

    return random_number + str(last_number)


def find_place(national_id: str) -> Union[dict, None]:
    if not validate(national_id):
        raise InvalidNationalId(national_id)

    code = convert_to_en(national_id[:3])
    places = [place for place in places_code if code in place['code']]

    if places:
        place = places[0].copy()
        place['code'] = place['code'].split('-')
        return place

    return None



class InvalidPhoneNumber(Exception):
    def __init__(self, phone_number):
        self.phone_number = phone_number
        self.message = phone_number + ' is an invalid phone number'
        super().__init__(self.message)



def parsian_account_number_calculator(sheba: str) -> dict:
    sheba = sheba[14:]
    formatted = '0' + sheba[:2] + '-0' + sheba[2:9] + '-' + sheba[9:]

    return {
        'normal': sheba,
        'formatted': formatted
    }


def pasargad_account_number_calculator(sheba: str) -> dict:
    sheba = sheba[7:]
    while sheba[0] == '0':
        sheba = sheba[1:]
    sheba = sheba[:len(sheba) - 2]
    formatted = '-'.join([sheba[:3], sheba[3:6], sheba[6:14], sheba[14:]])

    return {
        'normal': sheba,
        'formatted': formatted
    }


def shahr_account_number_calculator(sheba: str) -> dict:
    sheba = sheba[7:]
    while sheba[0] == '0':
        sheba = sheba[1:]

    return {
        'normal': sheba,
        'formatted': sheba
    }


data = [
    {
        'nickname': 'ayandeh',
        'name': 'Ayandeh Bank',
        'persian_name': 'بانک آینده',
        'card_prefix': ['636214'],
        'sheba_code': ['062'],
    },
    {
        'nickname': 'eghtesad-novin',
        'name': 'Eghtesad Novin Bank',
        'persian_name': 'بانک اقتصاد نوین',
        'card_prefix': ['627412'],
        'sheba_code': ['055'],
    },
    {
        'nickname': 'ansar',
        'name': 'Ansar Bank',
        'persian_name': 'بانک انصار',
        'card_prefix': ['627381'],
        'sheba_code': ['063'],
    },
    {
        'nickname': 'iran-zamin',
        'name': 'Iran Zamin Bank',
        'persian_name': 'بانک ایران زمین',
        'card_prefix': ['505785'],
        'sheba_code': ['069'],
    },
    {
        'nickname': 'parsian',
        'name': 'Parsian Bank',
        'persian_name': 'بانک پارسیان',
        'card_prefix': ['622106', '627884'],
        'sheba_code': ['054'],
        'account_number_calculator': parsian_account_number_calculator
    },
    {
        'nickname': 'pasargad',
        'name': 'Pasargad Bank',
        'persian_name': 'بانک پاسارگاد',
        'card_prefix': ['502229', '639347'],
        'sheba_code': ['057'],
        'account_number_calculator': pasargad_account_number_calculator
    },
    {
        'nickname': 'post',
        'name': 'Post Bank',
        'persian_name': 'پست بانک ایران',
        'card_prefix': ['627760'],
        'sheba_code': ['021'],
    },
    {
        'nickname': 'tejarat',
        'name': 'Tejarat Bank',
        'persian_name': 'بانک تجارت',
        'card_prefix': ['585983', '627353'],
        'sheba_code': ['018'],
    },
    {
        'nickname': 'toose-taavon',
        'name': 'Tosee Taavon Bank',
        'persian_name': 'بانک توسعه تعاون',
        'card_prefix': ['502908'],
        'sheba_code': ['022'],
    },
    {
        'nickname': 'tosee-saderat',
        'name': 'Tose Saderat Bank',
        'persian_name': 'بانک توسعه صادرات',
        'card_prefix': ['207177', '627648'],
        'sheba_code': ['020'],
    },
    {
        'nickname': 'hekmat-iranian',
        'name': 'Hekmat Iranian Bank',
        'persian_name': 'بانک حکمت ایرانیان',
        'card_prefix': ['636949'],
        'sheba_code': ['065'],
    },
    {
        'nickname': 'middle-east-bank',
        'name': 'Middle East Bank',
        'persian_name': 'بانک خاورمیانه',
        'card_prefix': ['585949'],
        'sheba_code': ['078'],
    },
    {
        'nickname': 'dey',
        'name': 'Dey Bank',
        'persian_name': 'بانک دی',
        'card_prefix': ['502938'],
        'sheba_code': ['066'],
    },
    {
        'nickname': 'resalat',
        'name': 'Resalat Bank',
        'persian_name': 'بانک رسالت',
        'card_prefix': ['504172'],
        'sheba_code': ['070'],
    },
    {
        'nickname': 'refah',
        'name': 'Refah Bank',
        'persian_name': 'بانک رفاه کارگران',
        'card_prefix': ['589463'],
        'sheba_code': ['013'],
    },
    {
        'nickname': 'saman',
        'name': 'Saman Bank',
        'persian_name': 'بانک سامان',
        'card_prefix': ['621986'],
        'sheba_code': ['056'],
    },
    {
        'nickname': 'sepah',
        'name': 'Sepah Bank',
        'persian_name': 'بانک سپه',
        'card_prefix': ['589210'],
        'sheba_code': ['015'],
    },
    {
        'nickname': 'sarmayeh',
        'name': 'Sarmayeh Bank',
        'persian_name': 'بانک سرمایه',
        'card_prefix': ['639607'],
        'sheba_code': ['058'],
    },
    {
        'nickname': 'sina',
        'name': 'Sina Bank',
        'persian_name': 'بانک سینا',
        'card_prefix': ['639346'],
        'sheba_code': ['059'],
    },
    {
        'nickname': 'shahr',
        'name': 'City Bank',
        'persian_name': 'بانک شهر',
        'card_prefix': ['502806', '504706'],
        'sheba_code': ['061'],
        'account_number_calculator': shahr_account_number_calculator
    },
    {
        'nickname': 'saderat',
        'name': 'Saderat Bank',
        'persian_name': 'بانک صادرات ایران',
        'card_prefix': ['603769', '903769'],
        'sheba_code': ['019'],
    },
    {
        'nickname': 'sanat-o-madan',
        'name': 'Sanat O Madan Bank',
        'persian_name': 'بانک صنعت و معدن',
        'card_prefix': ['627961'],
        'sheba_code': ['011'],
    },
    {
        'nickname': 'mehr-eqtesad',
        'name': 'Mehr Eqtesad Bank',
        'persian_name': 'بانک قرض الحسنه مهر',
        'card_prefix': ['639370'],
        'sheba_code': ['079'],
    },
    {
        'nickname': 'ghavamin',
        'name': 'Ghavamin Bank',
        'persian_name': 'بانک قوامین',
        'card_prefix': ['639599'],
        'sheba_code': ['052'],
    },
    {
        'nickname': 'karafarin',
        'name': 'Karafarin Bank',
        'persian_name': 'بانک کارآفرین',
        'card_prefix': ['627488'],
        'sheba_code': ['053'],
    },
    {
        'nickname': 'keshavarzi',
        'name': 'Keshavarzi',
        'persian_name': 'بانک کشاورزی',
        'card_prefix': ['603770', '639217'],
        'sheba_code': ['016'],
    },
    {
        'nickname': 'gardeshgari',
        'name': 'Gardeshgari Bank',
        'persian_name': 'بانک گردشگری',
        'card_prefix': ['505416', '505426'],
        'sheba_code': ['064'],
    },
    {
        'nickname': 'central-bank',
        'name': 'Central Bank of Iran',
        'persian_name': 'بانک مرکزی ایران',
        'card_prefix': ['636797'],
        'sheba_code': ['010'],
    },
    {
        'nickname': 'maskan',
        'name': 'Maskan Bank',
        'persian_name': 'بانک مسکن',
        'card_prefix': ['628023'],
        'sheba_code': ['014'],
    },
    {
        'nickname': 'mellat',
        'name': 'Mellat Bank',
        'persian_name': 'بانک ملت',
        'card_prefix': ['610433', '991975'],
        'sheba_code': ['012'],
    },
    {
        'nickname': 'melli',
        'name': 'Melli',
        'persian_name': 'بانک ملی ایران',
        'card_prefix': ['170019', '603799'],
        'sheba_code': ['017'],
    },
    {
        'nickname': 'mehr-iran',
        'name': 'Mehr Iran Bank',
        'persian_name': 'بانک مهر ایران',
        'card_prefix': ['606373'],
        'sheba_code': ['090', '060'],
    },
    {
        'nickname': 'kosar',
        'name': 'Kosar Credit Institute',
        'persian_name': 'موسسه کوثر',
        'card_prefix': ['505801'],
        'sheba_code': ['073'],
    },
    {
        'nickname': 'melal',
        'name': 'Melal Credit Institute',
        'persian_name': 'موسسه اعتباری ملل',
        'card_prefix': ['606256'],
        'sheba_code': ['075'],
    },
    {
        'nickname': 'tosee',
        'name': 'Tosee Bank',
        'persian_name': 'موسسه اعتباری توسعه',
        'card_prefix': ['628157'],
        'sheba_code': '051',
    },
    {
        'nickname': 'noor-bank',
        'name': 'Noor Credit Institution',
        'persian_name': 'موسسه اعتباری نور',
        'sheba_code': ['080'],
    },
    {
        'nickname': 'iran-venezuela',
        'name': 'Iran and Venezuela Bank',
        'persian_name': 'بانک ایران و ونزوئلا',
        'sheba_code': ['095'],
    },
]



class InvalidCardNumber(Exception):
    def __init__(self, card_number):
        self.card_number = card_number
        self.message = card_number + ' is an invalid card number'
        super().__init__(self.message)


class InvalidShebaNumber(Exception):
    def __init__(self, sheba):
        self.card_number = sheba
        self.message = sheba + ' is an invalid sheba number'
        super().__init__(self.message)



from persian_tools import national_id
from persian_tools.national_id.exceptions import InvalidNationalId
import pytest


def test_validate_national_id():
    # this national id is equal to 0068415941
    assert national_id.validate('068415941')

    # this national id is equal to 0123000000
    assert national_id.validate('123000000') is False
    assert national_id.validate('0000000000') is False
    assert national_id.validate('4271427685') is False
    assert national_id.validate('0200203041') is False
    assert national_id.validate('0684159415') is False
    assert national_id.validate('068415941d') is False
    assert national_id.validate('06841594156') is False

    assert national_id.validate('0499370899')
    assert national_id.validate('0790419904')
    assert national_id.validate('0084575948')
    assert national_id.validate('0963695398')
    assert national_id.validate('0684159414')

    assert national_id.validate('0067749828')
    assert national_id.validate('0650451252')
    assert national_id.validate('1583250689')
    assert national_id.validate('4032152314')
    assert national_id.validate('0076229645')


def test_generate_random_national_id():
    for _ in range(15):
        assert national_id.validate(national_id.generate_random())


def test_find_place_national_id():
    assert national_id.find_place('0499370899').get('city') == 'شهرری'
    assert national_id.find_place('0790419904').get('city') == 'سبزوار'
    assert national_id.find_place('0084575948').get('city') == 'تهران مرکزی'
    assert national_id.find_place('0060495219').get('city') == 'تهران مرکزی'
    assert national_id.find_place('0671658506').get('city') == 'بجنورد'
    assert national_id.find_place('0671658506').get('city') == 'بجنورد'
    assert national_id.find_place('0643005846').get('city') == 'بیرجند'
    assert national_id.find_place('0906582709').get('city') == 'کاشمر'
    assert national_id.find_place('0451727304').get('city') == 'شمیران'
    assert national_id.find_place('0371359058').get('city') == 'قم'
    assert national_id.find_place('5049478618').get('city') == 'پارس آباد'
    assert national_id.find_place('2110990147').get('city') == 'گرگان'

    assert national_id.find_place('9982675801') is None

    with pytest.raises(InvalidNationalId, match='.*is an invalid national id'):
        national_id.find_place('0084545943')
        national_id.find_place('008454594')
        national_id.find_place('8881234567')



from typing import Union
from math import floor
from .strings import SCALE, NUMBER_TEXT
from ..ordinal_suffix import add as add_ordinal_suffix


class LANGUAGES:
    EN = 'en'
    FA = 'fa'
    AR = 'ar'


SUPPORTED_CHARS = {
    LANGUAGES.EN: '0123456789',
    LANGUAGES.FA: '۰۱۲۳۴۵۶۷۸۹',
    LANGUAGES.AR: '٠١٢٣٤٥٦٧٨٩',
}


def _conversion(number: Union[int, float, str],
                destination_lang: str = LANGUAGES.FA) -> Union[str, None]:
    if destination_lang not in SUPPORTED_CHARS.keys():
        return None

    result = str(number)

    source_chars = SUPPORTED_CHARS.copy()
    destination_chars = source_chars.pop(destination_lang)
    for chars in source_chars.values():
        for i in range(10):
            result = result.replace(chars[i], destination_chars[i])

    return result


def convert_to_fa(number: Union[int, float, str]) -> Union[str, None]:
    return _conversion(number, LANGUAGES.FA)


def convert_to_ar(number: Union[int, float, str]) -> Union[str, None]:
    return _conversion(number, LANGUAGES.AR)


def convert_to_en(number: Union[int, float, str]) -> Union[str, None]:
    return _conversion(number, LANGUAGES.EN)


def convert_to_word(number: int, ordinal: bool = False) -> Union[str, None]:
    def to_word(num: int) -> str:
        res = ''
        for unit in [100, 10, 1]:
            if floor(num / unit) * unit != 0:
                if num in NUMBER_TEXT.keys():
                    res += NUMBER_TEXT[num]
                    break
                else:
                    res += NUMBER_TEXT[floor(num / unit) * unit] + ' و '
                    num %= unit

        return res

    if number == 0:
        return 'صفر'

    is_negative = number < 0
    number = abs(number)

    base = 1000
    result = []
    while number > 0:
        result.append(to_word(number % base))
        number = floor(number / base)

    if len(result) > 6:
        return None

    for i in range(len(result)):
        if result[i] != '':
            result[i] += ' ' + SCALE[i] + ' و '

    result = list(reversed(result))
    words = ''.join(result)

    while words.endswith(' و '):
        words = words[:-3]

    if is_negative:
        words = 'منفی ' + words

    words = words.strip()

    if ordinal:
        words = add_ordinal_suffix(words)

    return words



def add(number: str) -> str:
    if number.endswith('ی'):
        return number + ' اُم'
    if number.endswith('سه'):
        return number[:-2] + 'سوم'
    return number + 'م'


def remove(word: str) -> str:
    word = word.replace('مین', '')
    word = word.replace(' ام', '')
    word = word.replace(' اُم', '')

    if word.endswith('سوم'):
        return word[:-3] + 'سه'
    elif word.endswith('م'):
        return word[:-1]
    return word



class InvalidNationalId(Exception):
    def __init__(self, national_id):
        self.card_number = national_id
        self.message = national_id + ' is an invalid national id'
        super().__init__(self.message)



from persian_tools import phone_number
from persian_tools.phone_number.exceptions import InvalidPhoneNumber
import pytest


def test_validate():
    assert phone_number.validate('09022002580')
    assert phone_number.validate('09122002580')
    assert phone_number.validate('09322002580')
    assert phone_number.validate('09192002580')
    assert phone_number.validate('+989022002580')
    assert phone_number.validate('09022002580')
    assert phone_number.validate('989022002580')
    assert phone_number.validate('00989022002580')
    assert phone_number.validate('9022002580')

    assert phone_number.validate('09802002580') is False
    assert phone_number.validate('0980200') is False


def test_operator_data():
    actual = phone_number.operator_data('09022002580')
    expected = {'province': [], 'base': 'کشوری', 'type': ['permanent', 'credit'],'operator': 'ایرانسل'}

    assert all(v == actual[k] for k, v in expected.items())
    assert len(expected) == len(actual)

    actual = phone_number.operator_data('09981000000')
    expected = {'province': [], 'base': 'کشوری', 'type': ['credit'], 'operator': 'شاتل موبایل'}

    assert all(v == actual[k] for k, v in expected.items())
    assert len(expected) == len(actual)

    actual = phone_number.operator_data('09300880440')
    expected = {'province': [], 'base': 'کشوری', 'type': ['permanent', 'credit'], 'operator': 'ایرانسل'}

    assert all(v == actual[k] for k, v in expected.items())
    assert len(expected) == len(actual)

    with pytest.raises(InvalidPhoneNumber, match='.*is an invalid phone number'):
        phone_number.operator_data('+989990467927')



class NotEnoughArguments(Exception):
    def __init__(self):
        self.message = 'Arguments are not enough; we need `bill_id` and `payment_id` or at least, bill `barcode`'
        super().__init__(self.message)



from typing import Union
import re


def add(number: Union[int, float, str], separator: str = ',') -> str:
    number = str(number)
    parts = number.split('.')
    pattern = '(\\d)(?=(\\d{3})+(?!\\d))'
    repl = r'\1{sep}'.format(sep=separator)

    parts[0] = re.sub(pattern, repl, parts[0])
    return '.'.join(parts)


def remove(number: str, separator: str = ',') -> str:
    return str(number).replace(separator, '')



data = [
    {
        'code': '169',
        'city': 'آذرشهر',
        'province': 'آذربایجان شرقی'
    },
    {
        'code': '170',
        'city': 'اسکو',
        'province': 'آذربایجان شرقی'
    },
    {
        'code': '149-150',
        'city': 'اهر',
        'province': 'آذربایجان شرقی'
    },
    {
        'code': '171',
        'city': 'بستان آباد',
        'province': 'آذربایجان شرقی'
    },
    {
        'code': '168',
        'city': 'بناب',
        'province': 'آذربایجان شرقی'
    },
    {
        'code': '136-137-138',
        'city': 'تبریز',
        'province': 'آذربایجان شرقی'
    },
    {
        'code': '545',
        'city': 'ترکمانچای',
        'province': 'آذربایجان شرقی'
    },
    {
        'code': '505',
        'city': 'جلفا',
        'province': 'آذربایجان شرقی'
    },
    {
        'code': '636',
        'city': 'چاروایماق',
        'province': 'آذربایجان شرقی'
    },
    {
        'code': '164-165',
        'city': 'سراب',
        'province': 'آذربایجان شرقی'
    },
    {
        'code': '172',
        'city': 'شبستر',
        'province': 'آذربایجان شرقی'
    },
    {
        'code': '623',
        'city': 'صوفیان',
        'province': 'آذربایجان شرقی'
    },
    {
        'code': '506',
        'city': 'عجب شیر',
        'province': 'آذربایجان شرقی'
    },
    {
        'code': '519',
        'city': 'کلیبر',
        'province': 'آذربایجان شرقی'
    },
    {
        'code': '154-155',
        'city': 'مراغه',
        'province': 'آذربایجان شرقی'
    },
    {
        'code': '567',
        'city': 'ورزقان',
        'province': 'آذربایجان شرقی'
    },
    {
        'code': '173',
        'city': 'هریس',
        'province': 'آذربایجان شرقی'
    },
    {
        'code': '159-160',
        'city': 'هشترود',
        'province': 'آذربایجان شرقی'
    },
    {
        'code': '604',
        'city': 'هوراند',
        'province': 'آذربایجان شرقی'
    },
    {
        'code': '274-275',
        'city': 'ارومیه',
        'province': 'آذربایجان غربی'
    },
    {
        'code': '295',
        'city': 'اشنویه',
        'province': 'آذربایجان غربی'
    },
    {
        'code': '637',
        'city': 'انزل',
        'province': 'آذربایجان غربی'
    },
    {
        'code': '292',
        'city': 'بوکان',
        'province': 'آذربایجان غربی'
    },
    {
        'code': '492',
        'city': 'پلدشت',
        'province': 'آذربایجان غربی'
    },
    {
        'code': '289',
        'city': 'پیرانشهر',
        'province': 'آذربایجان غربی'
    },
    {
        'code': '677',
        'city': 'تخت سلیمان',
        'province': 'آذربایجان غربی'
    },
    {
        'code': '294',
        'city': 'تکاب',
        'province': 'آذربایجان غربی'
    },
    {
        'code': '493',
        'city': 'چایپاره',
        'province': 'آذربایجان غربی'
    },
    {
        'code': '279-280',
        'city': 'خوی',
        'province': 'آذربایجان غربی'
    },
    {
        'code': '288',
        'city': 'سردشت',
        'province': 'آذربایجان غربی'
    },
    {
        'code': '284-285',
        'city': 'سلماس',
        'province': 'آذربایجان غربی'
    },
    {
        'code': '638',
        'city': 'سیلوانه',
        'province': 'آذربایجان غربی'
    },
    {
        'code': '291',
        'city': 'سیه چشمه(چالدران)',
        'province': 'آذربایجان غربی'
    },
    {
        'code': '640',
        'city': 'شوط',
        'province': 'آذربایجان غربی'
    },
    {
        'code': '293',
        'city': 'شاهین دژ',
        'province': 'آذربایجان غربی'
    },
    {
        'code': '675',
        'city': 'کشاورز',
        'province': 'آذربایجان غربی'
    },
    {
        'code': '282-283',
        'city': 'ماکو',
        'province': 'آذربایجان غربی'
    },
    {
        'code': '286-287',
        'city': 'مهاباد',
        'province': 'آذربایجان غربی'
    },
    {
        'code': '296-297',
        'city': 'میاندوآب',
        'province': 'آذربایجان غربی'
    },
    {
        'code': '290',
        'city': 'نقده',
        'province': 'آذربایجان غربی'
    },
    {
        'code': '400-401',
        'city': 'اسدآباد',
        'province': 'همدان'
    },
    {
        'code': '404-405',
        'city': 'بهار',
        'province': 'همدان'
    },
    {
        'code': '397',
        'city': 'تویسرکان',
        'province': 'همدان'
    },
    {
        'code': '398-399',
        'city': 'رزن',
        'province': 'همدان'
    },
    {
        'code': '647',
        'city': 'شراء و پیشخوار',
        'province': 'همدان'
    },
    {
        'code': '502',
        'city': 'فامنین',
        'province': 'همدان'
    },
    {
        'code': '584',
        'city': 'قلقل رود',
        'province': 'همدان'
    },
    {
        'code': '402-403',
        'city': 'کبودرآهنگ',
        'province': 'همدان'
    },
    {
        'code': '392-393',
        'city': 'ملایر',
        'province': 'همدان'
    },
    {
        'code': '395-396',
        'city': 'نهاوند',
        'province': 'همدان'
    },
    {
        'code': '386-387',
        'city': 'همدان',
        'province': 'همدان'
    },
    {
        'code': '503',
        'city': 'ابرکوه',
        'province': 'یزد'
    },
    {
        'code': '444',
        'city': 'اردکان',
        'province': 'یزد'
    },
    {
        'code': '551',
        'city': 'اشکذر',
        'province': 'یزد'
    },
    {
        'code': '447',
        'city': 'بافق',
        'province': 'یزد'
    },
    {
        'code': '561',
        'city': 'بهاباد',
        'province': 'یزد'
    },
    {
        'code': '445',
        'city': 'تفت',
        'province': 'یزد'
    },
    {
        'code': '718',
        'city': 'دستگردان',
        'province': 'یزد'
    },
    {
        'code': '083',
        'city': 'طبس',
        'province': 'یزد'
    },
    {
        'code': '446',
        'city': 'مهریز',
        'province': 'یزد'
    },
    {
        'code': '448',
        'city': 'میبد',
        'province': 'یزد'
    },
    {
        'code': '552',
        'city': 'نیر',
        'province': 'یزد'
    },
    {
        'code': '543',
        'city': 'هرات و مروست',
        'province': 'یزد'
    },
    {
        'code': '442-443',
        'city': 'یزد',
        'province': 'یزد'
    },
    {
        'code': '051',
        'city': 'آشتیان',
        'province': 'مرکزی'
    },
    {
        'code': '052-053',
        'city': 'اراک',
        'province': 'مرکزی'
    },
    {
        'code': '058',
        'city': 'تفرش',
        'province': 'مرکزی'
    },
    {
        'code': '055',
        'city': 'خمین',
        'province': 'مرکزی'
    },
    {
        'code': '617',
        'city': 'خنداب',
        'province': 'مرکزی'
    },
    {
        'code': '057',
        'city': 'دلیجان',
        'province': 'مرکزی'
    },
    {
        'code': '618',
        'city': 'زرند مرکزی',
        'province': 'مرکزی'
    },
    {
        'code': '059-060',
        'city': 'ساوه',
        'province': 'مرکزی'
    },
    {
        'code': '061-062',
        'city': 'سربند',
        'province': 'مرکزی'
    },
    {
        'code': '544',
        'city': 'فراهان',
        'province': 'مرکزی'
    },
    {
        'code': '056',
        'city': 'محلات',
        'province': 'مرکزی'
    },
    {
        'code': '571',
        'city': 'وفس',
        'province': 'مرکزی'
    },
    {
        'code': '593',
        'city': 'هندودر',
        'province': 'مرکزی'
    },
    {
        'code': '667',
        'city': 'ابوموسی',
        'province': 'هرمزگان'
    },
    {
        'code': '348',
        'city': 'بستک',
        'province': 'هرمزگان'
    },
    {
        'code': '586',
        'city': 'بشاگرد',
        'province': 'هرمزگان'
    },
    {
        'code': '338-339',
        'city': 'بندرعباس',
        'province': 'هرمزگان'
    },
    {
        'code': '343-344',
        'city': 'بندرلنگه',
        'province': 'هرمزگان'
    },
    {
        'code': '346',
        'city': 'جاسک',
        'province': 'هرمزگان'
    },
    {
        'code': '337',
        'city': 'حاجی آباد',
        'province': 'هرمزگان'
    },
    {
        'code': '554',
        'city': 'خمیر',
        'province': 'هرمزگان'
    },
    {
        'code': '469',
        'city': 'رودان',
        'province': 'هرمزگان'
    },
    {
        'code': '537',
        'city': 'فین',
        'province': 'هرمزگان'
    },
    {
        'code': '345',
        'city': 'قشم',
        'province': 'هرمزگان'
    },
    {
        'code': '470',
        'city': 'گاوبندی',
        'province': 'هرمزگان'
    },
    {
        'code': '341-342',
        'city': 'میناب',
        'province': 'هرمزگان'
    },
    {
        'code': ' 483-484',
        'city': 'ازنا',
        'province': 'لرستان'
    },
    {
        'code': '557',
        'city': 'اشترینان',
        'province': 'لرستان'
    },
    {
        'code': '418',
        'city': 'الشتر',
        'province': 'لرستان'
    },
    {
        'code': '416-417',
        'city': 'الیگودرز',
        'province': 'لرستان'
    },
    {
        'code': '412-413',
        'city': 'بروجرد',
        'province': 'لرستان'
    },
    {
        'code': '592',
        'city': 'پاپی',
        'province': 'لرستان'
    },
    {
        'code': '612',
        'city': 'چغلوندی',
        'province': 'لرستان'
    },
    {
        'code': '613',
        'city': 'چگنی',
        'province': 'لرستان'
    },
    {
        'code': '406-407',
        'city': 'خرم آباد',
        'province': 'لرستان'
    },
    {
        'code': '421',
        'city': 'دورود',
        'province': 'لرستان'
    },
    {
        'code': '598',
        'city': 'رومشکان',
        'province': 'لرستان'
    },
    {
        'code': '419',
        'city': 'کوهدشت',
        'province': 'لرستان'
    },
    {
        'code': '385',
        'city': 'ملاوی(پلدختر)',
        'province': 'لرستان'
    },
    {
        'code': '420',
        'city': 'نورآباد(دلفان)',
        'province': 'لرستان'
    },
    {
        'code': '528',
        'city': 'ویسیان',
        'province': 'لرستان'
    },
    {
        'code': '213-214',
        'city': 'آمل',
        'province': 'مازندران'
    },
    {
        'code': '205-206',
        'city': 'بابل',
        'province': 'مازندران'
    },
    {
        'code': '498',
        'city': 'بابل',
        'province': 'مازندران'
    },
    {
        'code': '568',
        'city': 'بندپی',
        'province': 'مازندران'
    },
    {
        'code': '711',
        'city': 'بندپی شرقی',
        'province': 'مازندران'
    },
    {
        'code': '217-218',
        'city': 'بهشهر',
        'province': 'مازندران'
    },
    {
        'code': '221',
        'city': 'تنکابن',
        'province': 'مازندران'
    },
    {
        'code': '582',
        'city': 'جویبار',
        'province': 'مازندران'
    },
    {
        'code': '483',
        'city': 'چالوس',
        'province': 'مازندران'
    },
    {
        'code': '625',
        'city': 'چمستان',
        'province': 'مازندران'
    },
    {
        'code': '576',
        'city': 'چهاردانگه',
        'province': 'مازندران'
    },
    {
        'code': '578',
        'city': 'دودانگه',
        'province': 'مازندران'
    },
    {
        'code': '227',
        'city': 'رامسر',
        'province': 'مازندران'
    },
    {
        'code': '208-209',
        'city': 'ساری',
        'province': 'مازندران'
    },
    {
        'code': '225',
        'city': 'سوادکوه',
        'province': 'مازندران'
    },
    {
        'code': '577',
        'city': 'شیرگاه',
        'province': 'مازندران'
    },
    {
        'code': '712',
        'city': 'عباس آباد',
        'province': 'مازندران'
    },
    {
        'code': '215-216',
        'city': 'قائمشهر',
        'province': 'مازندران'
    },
    {
        'code': '626',
        'city': 'کجور',
        'province': 'مازندران'
    },
    {
        'code': '627',
        'city': 'کلاردشت',
        'province': 'مازندران'
    },
    {
        'code': '579',
        'city': 'گلوگاه',
        'province': 'مازندران'
    },
    {
        'code': '713',
        'city': 'میاندورود',
        'province': 'مازندران'
    },
    {
        'code': '499',
        'city': 'نکاء',
        'province': 'مازندران'
    },
    {
        'code': '222',
        'city': 'نور',
        'province': 'مازندران'
    },
    {
        'code': '219-220',
        'city': 'نوشهر',
        'province': 'مازندران'
    },
    {
        'code': '500-501',
        'city': 'هراز و محمودآباد',
        'province': 'مازندران'
    },
    {
        'code': '623',
        'city': 'آزادشهر',
        'province': 'گلستان'
    },
    {
        'code': '497',
        'city': 'آق قلا',
        'province': 'گلستان'
    },
    {
        'code': '223',
        'city': 'بندرترکمن',
        'province': 'گلستان'
    },
    {
        'code': '689',
        'city': 'بندرگز',
        'province': 'گلستان'
    },
    {
        'code': '487',
        'city': 'رامیان',
        'province': 'گلستان'
    },
    {
        'code': '226',
        'city': 'علی آباد',
        'province': 'گلستان'
    },
    {
        'code': '224',
        'city': 'کردکوی',
        'province': 'گلستان'
    },
    {
        'code': '386',
        'city': 'کلاله',
        'province': 'گلستان'
    },
    {
        'code': '211-212',
        'city': 'گرگان',
        'province': 'گلستان'
    },
    {
        'code': '628',
        'city': 'گمیشان',
        'province': 'گلستان'
    },
    {
        'code': '202-203',
        'city': 'گنبد کاووس',
        'province': 'گلستان'
    },
    {
        'code': '531',
        'city': 'مراوه تپه',
        'province': 'گلستان'
    },
    {
        'code': '288',
        'city': 'مینودشت',
        'province': 'گلستان'
    },
    {
        'code': '261',
        'city': 'آستارا',
        'province': 'گیلان'
    },
    {
        'code': '273',
        'city': 'آستانه',
        'province': 'گیلان'
    },
    {
        'code': '630',
        'city': 'املش',
        'province': 'گیلان'
    },
    {
        'code': '264',
        'city': 'بندرانزلی',
        'province': 'گیلان'
    },
    {
        'code': '518',
        'city': 'خمام',
        'province': 'گیلان'
    },
    {
        'code': '631',
        'city': 'رحیم آباد',
        'province': 'گیلان'
    },
    {
        'code': '258-259',
        'city': 'رشت',
        'province': 'گیلان'
    },
    {
        'code': '570',
        'city': 'رضوانشهر',
        'province': 'گیلان'
    },
    {
        'code': '265',
        'city': 'رودبار',
        'province': 'گیلان'
    },
    {
        'code': '268-269',
        'city': 'رودسر',
        'province': 'گیلان'
    },
    {
        'code': '653',
        'city': 'سنگر',
        'province': 'گیلان'
    },
    {
        'code': '517',
        'city': 'سیاهکل',
        'province': 'گیلان'
    },
    {
        'code': '569',
        'city': 'شفت',
        'province': 'گیلان'
    },
    {
        'code': '267',
        'city': 'صومعه سرا',
        'province': 'گیلان'
    },
    {
        'code': '262-263',
        'city': 'طالش',
        'province': 'گیلان'
    },
    {
        'code': '593',
        'city': 'عمارلو',
        'province': 'گیلان'
    },
    {
        'code': '266',
        'city': 'فومن',
        'province': 'گیلان'
    },
    {
        'code': '693',
        'city': 'کوچصفهان',
        'province': 'گیلان'
    },
    {
        'code': '271-272',
        'city': 'لاهیجان',
        'province': 'گیلان'
    },
    {
        'code': '694',
        'city': 'لشت نشاء',
        'province': 'گیلان'
    },
    {
        'code': '270',
        'city': 'لنگرود',
        'province': 'گیلان'
    },
    {
        'code': '516',
        'city': 'ماسال و شاندرمن',
        'province': 'گیلان'
    },
    {
        'code': '333-334',
        'city': 'اسلام آباد',
        'province': 'کرمانشاه'
    },
    {
        'code': '691',
        'city': 'باینگان',
        'province': 'کرمانشاه'
    },
    {
        'code': '323-322',
        'city': 'پاوه',
        'province': 'کرمانشاه'
    },
    {
        'code': '595',
        'city': 'ثلاث باباجانی',
        'province': 'کرمانشاه'
    },
    {
        'code': '395',
        'city': 'جوانرود',
        'province': 'کرمانشاه'
    },
    {
        'code': '641',
        'city': 'حمیل',
        'province': 'کرمانشاه'
    },
    {
        'code': '596',
        'city': 'روانسر',
        'province': 'کرمانشاه'
    },
    {
        'code': '336',
        'city': 'سرپل ذهاب',
        'province': 'کرمانشاه'
    },
    {
        'code': '335',
        'city': 'سنقر',
        'province': 'کرمانشاه'
    },
    {
        'code': '496',
        'city': 'صحنه',
        'province': 'کرمانشاه'
    },
    {
        'code': '337',
        'city': 'قصرشیرین',
        'province': 'کرمانشاه'
    },
    {
        'code': '324-325',
        'city': 'کرمانشاه',
        'province': 'کرمانشاه'
    },
    {
        'code': '394',
        'city': 'کرند',
        'province': 'کرمانشاه'
    },
    {
        'code': '330',
        'city': 'کنگاور',
        'province': 'کرمانشاه'
    },
    {
        'code': '332',
        'city': 'گیلانغرب',
        'province': 'کرمانشاه'
    },
    {
        'code': '331',
        'city': 'هرسین',
        'province': 'کرمانشاه'
    },
    {
        'code': '687',
        'city': 'باشت',
        'province': 'کهکیلویه و بویراحمد'
    },
    {
        'code': '422-423',
        'city': 'بویراحمد(یاسوج)',
        'province': 'کهکیلویه و بویراحمد'
    },
    {
        'code': '599',
        'city': 'بهمنی',
        'province': 'کهکیلویه و بویراحمد'
    },
    {
        'code': '600',
        'city': 'چاروسا',
        'province': 'کهکیلویه و بویراحمد'
    },
    {
        'code': '688',
        'city': 'دروهان',
        'province': 'کهکیلویه و بویراحمد'
    },
    {
        'code': '424-425',
        'city': 'کهکیلویه(دهدشت)',
        'province': 'کهکیلویه و بویراحمد'
    },
    {
        'code': '426',
        'city': 'گچساران(دوگنبدان)',
        'province': 'کهکیلویه و بویراحمد'
    },
    {
        'code': '550',
        'city': 'لنده',
        'province': 'کهکیلویه و بویراحمد'
    },
    {
        'code': '697',
        'city': 'مارگون',
        'province': 'کهکیلویه و بویراحمد'
    },
    {
        'code': '384',
        'city': 'بانه',
        'province': 'کردستان'
    },
    {
        'code': '377-378',
        'city': 'بیجار',
        'province': 'کردستان'
    },
    {
        'code': '558',
        'city': 'دهگلان',
        'province': 'کردستان'
    },
    {
        'code': '385',
        'city': 'دیواندره',
        'province': 'کردستان'
    },
    {
        'code': '646',
        'city': 'سروآباد',
        'province': 'کردستان'
    },
    {
        'code': '375-376',
        'city': 'سقز',
        'province': 'کردستان'
    },
    {
        'code': '372-373',
        'city': 'سنندج',
        'province': 'کردستان'
    },
    {
        'code': '379-380',
        'city': 'قروه',
        'province': 'کردستان'
    },
    {
        'code': '383',
        'city': 'کامیاران',
        'province': 'کردستان'
    },
    {
        'code': '674',
        'city': 'کرانی',
        'province': 'کردستان'
    },
    {
        'code': '381-382',
        'city': 'مریوان',
        'province': 'کردستان'
    },
    {
        'code': '676',
        'city': 'نمشیر',
        'province': 'کردستان'
    },
    {
        'code': '722',
        'city': 'ارزونیه',
        'province': 'کرمان'
    },
    {
        'code': '542',
        'city': 'انار',
        'province': 'کرمان'
    },
    {
        'code': '312-313',
        'city': 'بافت',
        'province': 'کرمان'
    },
    {
        'code': '317',
        'city': 'بردسیر',
        'province': 'کرمان'
    },
    {
        'code': '310-311',
        'city': 'بم',
        'province': 'کرمان'
    },
    {
        'code': '302-303',
        'city': 'جیرفت',
        'province': 'کرمان'
    },
    {
        'code': '583',
        'city': 'رابر',
        'province': 'کرمان'
    },
    {
        'code': '321',
        'city': 'راور',
        'province': 'کرمان'
    },
    {
        'code': '382',
        'city': 'راین',
        'province': 'کرمان'
    },
    {
        'code': '304-305',
        'city': 'رفسنجان',
        'province': 'کرمان'
    },
    {
        'code': '536',
        'city': 'رودبار کهنوج',
        'province': 'کرمان'
    },
    {
        'code': '605',
        'city': 'ریگان',
        'province': 'کرمان'
    },
    {
        'code': '308-309',
        'city': 'زرند',
        'province': 'کرمان'
    },
    {
        'code': '306-307',
        'city': 'سیرجان',
        'province': 'کرمان'
    },
    {
        'code': '319',
        'city': 'شهداد',
        'province': 'کرمان'
    },
    {
        'code': '313-314',
        'city': 'شهربابک',
        'province': 'کرمان'
    },
    {
        'code': '606',
        'city': 'عنبرآباد',
        'province': 'کرمان'
    },
    {
        'code': '320',
        'city': 'فهرج',
        'province': 'کرمان'
    },
    {
        'code': '698',
        'city': 'قلعه گنج',
        'province': 'کرمان'
    },
    {
        'code': '298-299',
        'city': 'کرمان',
        'province': 'کرمان'
    },
    {
        'code': '535',
        'city': 'کوهبنان',
        'province': 'کرمان'
    },
    {
        'code': '315-316',
        'city': 'کهنوج',
        'province': 'کرمان'
    },
    {
        'code': '318',
        'city': 'گلباف',
        'province': 'کرمان'
    },
    {
        'code': '607',
        'city': 'ماهان',
        'province': 'کرمان'
    },
    {
        'code': '608',
        'city': 'منوجان',
        'province': 'کرمان'
    },
    {
        'code': '508',
        'city': 'آبیک',
        'province': 'قزوین'
    },
    {
        'code': '538',
        'city': 'آوج',
        'province': 'قزوین'
    },
    {
        'code': '728',
        'city': 'البرز',
        'province': 'قزوین'
    },
    {
        'code': '509',
        'city': 'بوئین زهرا',
        'province': 'قزوین'
    },
    {
        'code': '438-439',
        'city': 'تاکستان',
        'province': 'قزوین'
    },
    {
        'code': '580',
        'city': 'رودبار الموت',
        'province': 'قزوین'
    },
    {
        'code': '590',
        'city': 'رودبار شهرستان',
        'province': 'قزوین'
    },
    {
        'code': '559',
        'city': 'ضیاءآباد',
        'province': 'قزوین'
    },
    {
        'code': '588',
        'city': 'طارم سفلی',
        'province': 'قزوین'
    },
    {
        'code': '431-432',
        'city': 'قزوین',
        'province': 'قزوین'
    },
    {
        'code': '037-038',
        'city': 'قم',
        'province': 'قم'
    },
    {
        'code': '702',
        'city': 'کهک',
        'province': 'قم'
    },
    {
        'code': '240-241',
        'city': 'آباده',
        'province': 'فارس'
    },
    {
        'code': '670',
        'city': 'آباده طشک',
        'province': 'فارس'
    },
    {
        'code': '648',
        'city': 'ارسنجان',
        'province': 'فارس'
    },
    {
        'code': '252',
        'city': 'استهبان',
        'province': 'فارس'
    },
    {
        'code': '678',
        'city': 'اشکنان',
        'province': 'فارس'
    },
    {
        'code': '253',
        'city': 'اقلید',
        'province': 'فارس'
    },
    {
        'code': '649',
        'city': 'اوز',
        'province': 'فارس'
    },
    {
        'code': '513',
        'city': 'بوانات',
        'province': 'فارس'
    },
    {
        'code': '546',
        'city': 'بیضا',
        'province': 'فارس'
    },
    {
        'code': '671',
        'city': 'جویم',
        'province': 'فارس'
    },
    {
        'code': '246-247',
        'city': 'جهرم',
        'province': 'فارس'
    },
    {
        'code': '654',
        'city': 'حاجی آباد(زرین دشت)',
        'province': 'فارس'
    },
    {
        'code': '548',
        'city': 'خرامه',
        'province': 'فارس'
    },
    {
        'code': '547',
        'city': 'خشت و کمارج',
        'province': 'فارس'
    },
    {
        'code': '655',
        'city': 'خفر',
        'province': 'فارس'
    },
    {
        'code': '248-249',
        'city': 'داراب',
        'province': 'فارس'
    },
    {
        'code': '253',
        'city': 'سپیدان',
        'province': 'فارس'
    },
    {
        'code': '514',
        'city': 'سروستان',
        'province': 'فارس'
    },
    {
        'code': '665',
        'city': 'سعادت آباد',
        'province': 'فارس'
    },
    {
        'code': '673',
        'city': 'شیبکوه',
        'province': 'فارس'
    },
    {
        'code': '228-229-230',
        'city': 'شیراز',
        'province': 'فارس'
    },
    {
        'code': '679',
        'city': 'فراشبند',
        'province': 'فارس'
    },
    {
        'code': '256-257',
        'city': 'فسا',
        'province': 'فارس'
    },
    {
        'code': '244-245',
        'city': 'فیروزآباد',
        'province': 'فارس'
    },
    {
        'code': '681',
        'city': 'قنقری(خرم بید)',
        'province': 'فارس'
    },
    {
        'code': '723',
        'city': 'قیروکارزین',
        'province': 'فارس'
    },
    {
        'code': '236-237',
        'city': 'کازرون',
        'province': 'فارس'
    },
    {
        'code': '683',
        'city': 'کوار',
        'province': 'فارس'
    },
    {
        'code': '656',
        'city': 'کراش',
        'province': 'فارس'
    },
    {
        'code': '250-251',
        'city': 'لارستان',
        'province': 'فارس'
    },
    {
        'code': '515',
        'city': 'لامرد',
        'province': 'فارس'
    },
    {
        'code': '242-243',
        'city': 'مرودشت',
        'province': 'فارس'
    },
    {
        'code': '238-239',
        'city': 'ممسنی',
        'province': 'فارس'
    },
    {
        'code': '657',
        'city': 'مهر',
        'province': 'فارس'
    },
    {
        'code': '255',
        'city': 'نی ریز',
        'province': 'فارس'
    },
    {
        'code': '684',
        'city': 'ایوانکی',
        'province': 'سمنان'
    },
    {
        'code': '700',
        'city': 'بسطام',
        'province': 'سمنان'
    },
    {
        'code': '642',
        'city': 'بیارجمند',
        'province': 'سمنان'
    },
    {
        'code': '457',
        'city': 'دامغان',
        'province': 'سمنان'
    },
    {
        'code': '456',
        'city': 'سمنان',
        'province': 'سمنان'
    },
    {
        'code': '458-459',
        'city': 'شاهرود',
        'province': 'سمنان'
    },
    {
        'code': '460',
        'city': 'گرمسار',
        'province': 'سمنان'
    },
    {
        'code': '530',
        'city': 'مهدیشهر',
        'province': 'سمنان'
    },
    {
        'code': '520',
        'city': 'میامی',
        'province': 'سمنان'
    },
    {
        'code': '358-359',
        'city': 'ایرانشهر',
        'province': 'سیستان و بلوچستان'
    },
    {
        'code': '682',
        'city': 'بزمان',
        'province': 'سیستان و بلوچستان'
    },
    {
        'code': '703',
        'city': 'بمپور',
        'province': 'سیستان و بلوچستان'
    },
    {
        'code': '364-365',
        'city': 'چابهار',
        'province': 'سیستان و بلوچستان'
    },
    {
        'code': '371',
        'city': 'خاش',
        'province': 'سیستان و بلوچستان'
    },
    {
        'code': '701',
        'city': 'دشتیاری',
        'province': 'سیستان و بلوچستان'
    },
    {
        'code': '720',
        'city': 'راسک',
        'province': 'سیستان و بلوچستان'
    },
    {
        'code': '366-367',
        'city': 'زابل',
        'province': 'سیستان و بلوچستان'
    },
    {
        'code': '704',
        'city': 'زابلی',
        'province': 'سیستان و بلوچستان'
    },
    {
        'code': '361-362',
        'city': 'زاهدان',
        'province': 'سیستان و بلوچستان'
    },
    {
        'code': '369-370',
        'city': 'سراوان',
        'province': 'سیستان و بلوچستان'
    },
    {
        'code': '635',
        'city': 'سرباز',
        'province': 'سیستان و بلوچستان'
    },
    {
        'code': '668',
        'city': 'سیب و سوران',
        'province': 'سیستان و بلوچستان'
    },
    {
        'code': '533',
        'city': 'شهرکی و ناروئی(زهک)',
        'province': 'سیستان و بلوچستان'
    },
    {
        'code': '705',
        'city': 'شیب آب',
        'province': 'سیستان و بلوچستان'
    },
    {
        'code': '699',
        'city': 'فنوج',
        'province': 'سیستان و بلوچستان'
    },
    {
        'code': '669',
        'city': 'قصرقند',
        'province': 'سیستان و بلوچستان'
    },
    {
        'code': '725',
        'city': 'کنارک',
        'province': 'سیستان و بلوچستان'
    },
    {
        'code': '597',
        'city': 'لاشار(اسپکه)',
        'province': 'سیستان و بلوچستان'
    },
    {
        'code': '611',
        'city': 'میرجاوه',
        'province': 'سیستان و بلوچستان'
    },
    {
        'code': '525',
        'city': 'نیک شهر',
        'province': 'سیستان و بلوچستان'
    },
    {
        'code': '181',
        'city': 'آبادان',
        'province': 'خوزستان'
    },
    {
        'code': '527',
        'city': 'آغاجاری',
        'province': 'خوزستان'
    },
    {
        'code': '585',
        'city': 'اروندکنار',
        'province': 'خوزستان'
    },
    {
        'code': '685',
        'city': 'امیدیه',
        'province': 'خوزستان'
    },
    {
        'code': '663',
        'city': 'اندیکا',
        'province': 'خوزستان'
    },
    {
        'code': '192-193',
        'city': 'اندیمشک',
        'province': 'خوزستان'
    },
    {
        'code': '174-175',
        'city': 'اهواز',
        'province': 'خوزستان'
    },
    {
        'code': '183-184',
        'city': 'ایذه',
        'province': 'خوزستان'
    },
    {
        'code': '481',
        'city': 'باغ ملک',
        'province': 'خوزستان'
    },
    {
        'code': '706',
        'city': 'بندر امام خمینی',
        'province': 'خوزستان'
    },
    {
        'code': '194-195',
        'city': 'بندرماهشهر',
        'province': 'خوزستان'
    },
    {
        'code': '185-186',
        'city': 'بهبهان',
        'province': 'خوزستان'
    },
    {
        'code': '182',
        'city': 'خرمشهر',
        'province': 'خوزستان'
    },
    {
        'code': '199-200',
        'city': 'دزفول',
        'province': 'خوزستان'
    },
    {
        'code': '198',
        'city': 'دشت آزادگان',
        'province': 'خوزستان'
    },
    {
        'code': '662',
        'city': 'رامشیر',
        'province': 'خوزستان'
    },
    {
        'code': '190-191',
        'city': 'رامهرمز',
        'province': 'خوزستان'
    },
    {
        'code': '692',
        'city': 'سردشت',
        'province': 'خوزستان'
    },
    {
        'code': '189',
        'city': 'شادگان',
        'province': 'خوزستان'
    },
    {
        'code': '707',
        'city': 'شاوور',
        'province': 'خوزستان'
    },
    {
        'code': '526',
        'city': 'شوش',
        'province': 'خوزستان'
    },
    {
        'code': '187-188',
        'city': 'شوشتر',
        'province': 'خوزستان'
    },
    {
        'code': '729',
        'city': 'گتوند',
        'province': 'خوزستان'
    },
    {
        'code': '730',
        'city': 'لالی',
        'province': 'خوزستان'
    },
    {
        'code': '196-197',
        'city': 'مسجدسلیمان',
        'province': 'خوزستان'
    },
    {
        'code': '661',
        'city': 'هندیجان',
        'province': 'خوزستان'
    },
    {
        'code': '680',
        'city': 'هویزه',
        'province': 'خوزستان'
    },
    {
        'code': '643',
        'city': 'احمدآباد',
        'province': 'خراسان رضوی'
    },
    {
        'code': '562',
        'city': 'بجستان',
        'province': 'خراسان رضوی'
    },
    {
        'code': '572',
        'city': 'بردسکن',
        'province': 'خراسان رضوی'
    },
    {
        'code': '074',
        'city': 'تایباد',
        'province': 'خراسان رضوی'
    },
    {
        'code': '644',
        'city': 'تخت جلگه',
        'province': 'خراسان رضوی'
    },
    {
        'code': '072-073',
        'city': 'تربت جام',
        'province': 'خراسان رضوی'
    },
    {
        'code': '069-070',
        'city': 'تربت حیدریه',
        'province': 'خراسان رضوی'
    },
    {
        'code': '521',
        'city': 'جغتای',
        'province': 'خراسان رضوی'
    },
    {
        'code': '573',
        'city': 'جوین',
        'province': 'خراسان رضوی'
    },
    {
        'code': '522',
        'city': 'چناران',
        'province': 'خراسان رضوی'
    },
    {
        'code': '724',
        'city': 'خلیل آباد',
        'province': 'خراسان رضوی'
    },
    {
        'code': '076',
        'city': 'خواف',
        'province': 'خراسان رضوی'
    },
    {
        'code': '077',
        'city': 'درگز',
        'province': 'خراسان رضوی'
    },
    {
        'code': '650',
        'city': 'رشتخوار',
        'province': 'خراسان رضوی'
    },
    {
        'code': '574',
        'city': 'زبرخان',
        'province': 'خراسان رضوی'
    },
    {
        'code': '078-079',
        'city': 'سبزوار',
        'province': 'خراسان رضوی'
    },
    {
        'code': '081',
        'city': 'سرخس',
        'province': 'خراسان رضوی'
    },
    {
        'code': '084',
        'city': 'فریمان',
        'province': 'خراسان رضوی'
    },
    {
        'code': '651',
        'city': 'فیض آباد',
        'province': 'خراسان رضوی'
    },
    {
        'code': '086-087',
        'city': 'قوچان',
        'province': 'خراسان رضوی'
    },
    {
        'code': '089-090',
        'city': 'کاشمر',
        'province': 'خراسان رضوی'
    },
    {
        'code': '553',
        'city': 'کلات',
        'province': 'خراسان رضوی'
    },
    {
        'code': '091',
        'city': 'گناباد',
        'province': 'خراسان رضوی'
    },
    {
        'code': '092-093-094',
        'city': 'مشهد',
        'province': 'خراسان رضوی'
    },
    {
        'code': '097',
        'city': 'مشهد منطقه2',
        'province': 'خراسان رضوی'
    },
    {
        'code': '098',
        'city': 'مشهد منطقه3',
        'province': 'خراسان رضوی'
    },
    {
        'code': '096',
        'city': 'مشهد منطقه1',
        'province': 'خراسان رضوی'
    },
    {
        'code': '105-106',
        'city': 'نیشابور',
        'province': 'خراسان رضوی'
    },
    {
        'code': '063',
        'city': 'اسفراین',
        'province': 'خراسان شمالی'
    },
    {
        'code': '067-068',
        'city': 'بجنورد',
        'province': 'خراسان شمالی'
    },
    {
        'code': '075',
        'city': 'جاجرم',
        'province': 'خراسان شمالی'
    },
    {
        'code': '591',
        'city': 'رازوجرکلان',
        'province': 'خراسان شمالی'
    },
    {
        'code': '082',
        'city': 'شیروان',
        'province': 'خراسان شمالی'
    },
    {
        'code': '635',
        'city': 'فاروج',
        'province': 'خراسان شمالی'
    },
    {
        'code': '524',
        'city': 'مانه و سملقان',
        'province': 'خراسان شمالی'
    },
    {
        'code': '468',
        'city': 'اردل',
        'province': 'چهارمحال و بختیاری'
    },
    {
        'code': '465',
        'city': 'بروجن',
        'province': 'چهارمحال و بختیاری'
    },
    {
        'code': '461-462',
        'city': 'شهرکرد',
        'province': 'چهارمحال و بختیاری'
    },
    {
        'code': '467',
        'city': 'فارسان',
        'province': 'چهارمحال و بختیاری'
    },
    {
        'code': '632',
        'city': 'فلارد',
        'province': 'چهارمحال و بختیاری'
    },
    {
        'code': '555',
        'city': 'کوهرنگ',
        'province': 'چهارمحال و بختیاری'
    },
    {
        'code': '633',
        'city': 'کیار',
        'province': 'چهارمحال و بختیاری'
    },
    {
        'code': '629',
        'city': 'گندمان',
        'province': 'چهارمحال و بختیاری'
    },
    {
        'code': '466',
        'city': 'لردگان',
        'province': 'چهارمحال و بختیاری'
    },
    {
        'code': '696',
        'city': 'میانکوه',
        'province': 'چهارمحال و بختیاری'
    },
    {
        'code': '721',
        'city': 'بشرویه',
        'province': 'خراسان جنوبی'
    },
    {
        'code': '064-065',
        'city': 'بیرجند',
        'province': 'خراسان جنوبی'
    },
    {
        'code': '523',
        'city': 'درمیان',
        'province': 'خراسان جنوبی'
    },
    {
        'code': '652',
        'city': 'زیرکوه',
        'province': 'خراسان جنوبی'
    },
    {
        'code': '719',
        'city': 'سرایان',
        'province': 'خراسان جنوبی'
    },
    {
        'code': '716',
        'city': 'سربیشه',
        'province': 'خراسان جنوبی'
    },
    {
        'code': '085',
        'city': 'فردوس',
        'province': 'خراسان جنوبی'
    },
    {
        'code': '088',
        'city': 'قائنات',
        'province': 'خراسان جنوبی'
    },
    {
        'code': '563',
        'city': 'نهبندان',
        'province': 'خراسان جنوبی'
    },
    {
        'code': '529',
        'city': 'بندر دیلم',
        'province': 'بوشهر'
    },
    {
        'code': '353',
        'city': 'بندر گناوه',
        'province': 'بوشهر'
    },
    {
        'code': '349-350',
        'city': 'بوشهر',
        'province': 'بوشهر'
    },
    {
        'code': '355',
        'city': 'تنگستان',
        'province': 'بوشهر'
    },
    {
        'code': '609',
        'city': 'جم',
        'province': 'بوشهر'
    },
    {
        'code': '351-352',
        'city': 'دشتستان',
        'province': 'بوشهر'
    },
    {
        'code': '354',
        'city': 'دشتی',
        'province': 'بوشهر'
    },
    {
        'code': '732',
        'city': 'دلوار',
        'province': 'بوشهر'
    },
    {
        'code': '357',
        'city': 'دیر',
        'province': 'بوشهر'
    },
    {
        'code': '532',
        'city': 'سعد آباد',
        'province': 'بوشهر'
    },
    {
        'code': '610',
        'city': 'شبانکاره',
        'province': 'بوشهر'
    },
    {
        'code': '356',
        'city': 'کنگان',
        'province': 'بوشهر'
    },
    {
        'code': '556',
        'city': 'اسلامشهر',
        'province': 'تهران'
    },
    {
        'code': '658',
        'city': 'پاکدشت',
        'province': 'تهران'
    },
    {
        'code': '001-002-003-004-005-006-007-008',
        'city': 'تهران مرکزی',
        'province': 'تهران'
    },
    {
        'code': '011',
        'city': 'تهران جنوب',
        'province': 'تهران'
    },
    {
        'code': '020',
        'city': 'تهران شرق',
        'province': 'تهران'
    },
    {
        'code': '025',
        'city': 'تهرانشمال',
        'province': 'تهران'
    },
    {
        'code': '015',
        'city': 'تهران غرب',
        'province': 'تهران'
    },
    {
        'code': '043',
        'city': 'دماوند',
        'province': 'تهران'
    },
    {
        'code': '666',
        'city': 'رباط کریم',
        'province': 'تهران'
    },
    {
        'code': '489',
        'city': 'ساوجبلاغ',
        'province': 'تهران'
    },
    {
        'code': '044-045',
        'city': 'شمیران',
        'province': 'تهران'
    },
    {
        'code': '048-049',
        'city': 'شهرری',
        'province': 'تهران'
    },
    {
        'code': '490-491',
        'city': 'شهریار',
        'province': 'تهران'
    },
    {
        'code': '695',
        'city': 'طالقان',
        'province': 'تهران'
    },
    {
        'code': '659',
        'city': 'فیروزکوه',
        'province': 'تهران'
    },
    {
        'code': '031-032',
        'city': 'کرج',
        'province': 'تهران'
    },
    {
        'code': '664',
        'city': 'کهریزک',
        'province': 'تهران'
    },
    {
        'code': '717',
        'city': 'نظرآباد',
        'province': 'تهران'
    },
    {
        'code': '041-042',
        'city': 'ورامین',
        'province': 'تهران'
    },
    {
        'code': '471-472',
        'city': 'امور خارجه',
        'province': 'امور خارجه'
    },
    {
        'code': '454',
        'city': 'آبدانان',
        'province': 'ایلام'
    },
    {
        'code': '581',
        'city': 'ارکوازی(ملکشاهی)',
        'province': 'ایلام'
    },
    {
        'code': '449-450',
        'city': 'ایلام',
        'province': 'ایلام'
    },
    {
        'code': '616',
        'city': 'ایوان',
        'province': 'ایلام'
    },
    {
        'code': '534',
        'city': 'بدره',
        'province': 'ایلام'
    },
    {
        'code': '455',
        'city': 'دره شهر',
        'province': 'ایلام'
    },
    {
        'code': '451',
        'city': 'دهلران',
        'province': 'ایلام'
    },
    {
        'code': '726',
        'city': 'زرین آباد',
        'province': 'ایلام'
    },
    {
        'code': '634',
        'city': 'شیروان لومار',
        'province': 'ایلام'
    },
    {
        'code': '453',
        'city': 'شیروان و چرداول',
        'province': 'ایلام'
    },
    {
        'code': '727',
        'city': 'موسیان',
        'province': 'ایلام'
    },
    {
        'code': '452',
        'city': 'مهران',
        'province': 'ایلام'
    },
    {
        'code': '145-146',
        'city': 'اردبیل',
        'province': 'اردبیل'
    },
    {
        'code': '731',
        'city': 'ارشق',
        'province': 'اردبیل'
    },
    {
        'code': '690',
        'city': 'انگوت',
        'province': 'اردبیل'
    },
    {
        'code': '601',
        'city': 'بیله سوار',
        'province': 'اردبیل'
    },
    {
        'code': '504',
        'city': 'پارس آباد',
        'province': 'اردبیل'
    },
    {
        'code': '163',
        'city': 'خلخال',
        'province': 'اردبیل'
    },
    {
        'code': '714',
        'city': 'خورش رستم',
        'province': 'اردبیل'
    },
    {
        'code': '715',
        'city': 'سرعین',
        'province': 'اردبیل'
    },
    {
        'code': '566',
        'city': 'سنجبد(کوثر)',
        'province': 'اردبیل'
    },
    {
        'code': '166-167',
        'city': 'مشکین شهر',
        'province': 'اردبیل'
    },
    {
        'code': '161-162',
        'city': 'مغان',
        'province': 'اردبیل'
    },
    {
        'code': '686',
        'city': 'نمین',
        'province': 'اردبیل'
    },
    {
        'code': '603',
        'city': 'نیر',
        'province': 'اردبیل'
    },
    {
        'code': '619',
        'city': 'آران و بیدگل',
        'province': 'اصفهان'
    },
    {
        'code': '118',
        'city': 'اردستان',
        'province': 'اصفهان'
    },
    {
        'code': '127-128-129',
        'city': 'اصفهان',
        'province': 'اصفهان'
    },
    {
        'code': '620',
        'city': 'باغ بهادران',
        'province': 'اصفهان'
    },
    {
        'code': '621',
        'city': 'بوئین و میاندشت',
        'province': 'اصفهان'
    },
    {
        'code': '549',
        'city': 'تیران و کرون',
        'province': 'اصفهان'
    },
    {
        'code': '564',
        'city': 'جرقویه',
        'province': 'اصفهان'
    },
    {
        'code': '575',
        'city': 'چادگان',
        'province': 'اصفهان'
    },
    {
        'code': '113-114',
        'city': 'خمینی شهر',
        'province': 'اصفهان'
    },
    {
        'code': '122',
        'city': 'خوانسار',
        'province': 'اصفهان'
    },
    {
        'code': '540',
        'city': 'خور و بیابانک',
        'province': 'اصفهان'
    },
    {
        'code': '660',
        'city': 'دولت آباد',
        'province': 'اصفهان'
    },
    {
        'code': '120',
        'city': 'سمیرم',
        'province': 'اصفهان'
    },
    {
        'code': '512',
        'city': 'سمیرم سفلی (دهاقان)',
        'province': 'اصفهان'
    },
    {
        'code': '510-511',
        'city': 'شاهین شهر',
        'province': 'اصفهان'
    },
    {
        'code': '119',
        'city': 'شهرضا',
        'province': 'اصفهان'
    },
    {
        'code': '115',
        'city': 'فریدن',
        'province': 'اصفهان'
    },
    {
        'code': '112',
        'city': 'فریدونشهر',
        'province': 'اصفهان'
    },
    {
        'code': '110-111',
        'city': 'فلاورجان',
        'province': 'اصفهان'
    },
    {
        'code': '125-126',
        'city': 'کاشان',
        'province': 'اصفهان'
    },
    {
        'code': '565',
        'city': 'کوهپایه',
        'province': 'اصفهان'
    },
    {
        'code': '121',
        'city': 'گلپایگان',
        'province': 'اصفهان'
    },
    {
        'code': '116-117',
        'city': 'لنجان(زرینشهر)',
        'province': 'اصفهان'
    },
    {
        'code': '541',
        'city': 'مبارکه',
        'province': 'اصفهان'
    },
    {
        'code': '622',
        'city': 'میمه',
        'province': 'اصفهان'
    },
    {
        'code': '124',
        'city': 'نائین',
        'province': 'اصفهان'
    },
    {
        'code': '108-109',
        'city': 'نجف آباد',
        'province': 'اصفهان'
    },
    {
        'code': '123',
        'city': 'نطنز',
        'province': 'اصفهان'
    },
    {
        'code': '428-427',
        'city': 'زنجان',
        'province': 'اصفهان'
    },
    {
        'code': '507',
        'city': 'ملکان',
        'province': 'اصفهان'
    },
    {
        'code': '158',
        'city': 'مرند',
        'province': 'اصفهان'
    },
    {
        'code': '615',
        'city': 'ابهر',
        'province': 'اصفهان'
    },
    {
        'code': '615',
        'city': 'خرمدره',
        'province': 'اصفهان'
    },
    {
        'code': '152-153',
        'city': 'میانه',
        'province': 'اصفهان'
    },
]



from typing import Union
from .exceptions import NotEnoughArguments

TYPES = {
    1: "آب",
    2: "برق",
    3: "گاز",
    4: "تلفن ثابت",
    5: "تلفن همراه",
    6: "عوارض شهرداری",
    8: "سازمان مالیات",
    9: "جرایم راهنمایی و رانندگی",
}

CURRENCY_TOMAN = 100
CURRENCY_RIAL = 1000


def _cal_the_bit(num: str) -> int:
    _sum = 0
    base = 2

    for i in range(len(num)):
        if base == 8:
            base = 2

        sub_str = num[-1 - i]
        _sum += int(sub_str) * base

        base += 1

    _sum %= 11
    if _sum < 2:
        return 0
    else:
        return 11 - _sum


def get_detail(bill_id: Union[int, None] = None, payment_id: Union[int, None] = None, barcode: Union[str, None] = None,
               currency: str = CURRENCY_TOMAN) -> dict:
    if bill_id and payment_id:
        barcode = str(bill_id) + '000' + str(payment_id)
    elif barcode:
        bill_id = int(barcode[:13])
        payment_id = int(barcode[16:26])
    else:
        raise NotEnoughArguments()

    def verification_payment_id():
        str_bill_id = str(bill_id)
        str_payment_id = str(payment_id)

        if not str_payment_id or len(str_payment_id) < 6:
            return False

        control_bit1 = str_payment_id[len(str_payment_id) - 2]
        control_bit2 = str_payment_id[len(str_payment_id) - 1]
        control_payment_id = str_payment_id[:len(str_payment_id) - 2]

        return _cal_the_bit(control_payment_id) == int(control_bit1) and \
               _cal_the_bit(str_bill_id + control_payment_id + control_bit1) == int(control_bit2)

    valid_payment_id = verification_payment_id()

    amount = None
    if valid_payment_id:
        amount = int(str(payment_id)[:-5]) * currency

    bill_type = TYPES.get(int(str(bill_id)[-2:-1]))

    def verification_bill_id():
        str_bill_id = str(bill_id)

        if not str_bill_id or len(str_bill_id) < 6:
            return False

        control_bit = str_bill_id[-1]

        new_str_bill_id = str_bill_id[:-1]
        return _cal_the_bit(new_str_bill_id) == int(control_bit) and bill_type is not None

    valid_bill_id = verification_bill_id()

    valid_bill_data = valid_bill_id and valid_payment_id

    return {
        'amount': amount,
        'type': bill_type,
        'barcode': barcode,
        'bill_id': bill_id,
        'payment_id': payment_id,
        'is_valid': valid_bill_data,
        'is_valid_bill_id': valid_bill_id,
        'is_valid_payment_id': valid_payment_id,
    }



import re
from typing import Union
from .exceptions import InvalidCardNumber
from .banks_code import data as banks_code
from ..digits import convert_to_en


def validate(card_number: str) -> bool:
    if not card_number.isdigit() or \
            len(card_number) != 16 or \
            int(card_number[1:10]) == 0 or \
            int(card_number[10:]) == 0:
        return False

    _sum = 0
    for i in range(16):
        ratio = 2 if i % 2 == 0 else 1
        res = int(card_number[i]) * ratio
        _sum += res - 9 if res > 9 else res

    return _sum % 10 == 0


def bank_data(card_number: str) -> Union[dict, None]:
    if not validate(card_number):
        raise InvalidCardNumber(card_number)

    card_prefix = card_number[:6]
    bank = [bank for bank in banks_code if card_prefix in bank.get('card_prefix', [])]

    if len(bank) < 1:
        return None

    data = bank[0].copy()
    if data.get('account_number_calculator'):
        del data['account_number_calculator']
    return data


def extract_card_numbers(text: str, check_validation: bool = True, detect_bank_name: bool = False,
                         filter_valid_card_numbers: bool = True):
    card_number_regex = r'([\u06F0-\u06F90-9-_.*]{16,20})'
    results = []

    for i, matched_card_number in enumerate(re.findall(card_number_regex, text, re.MULTILINE)):
        card_number = convert_to_en(matched_card_number)

        for sep in ['-', '_', '.', '*']:
            if sep in matched_card_number:
                card_number = card_number.replace(sep, '')

        result = {
            'index': i + 1,
            'base': matched_card_number,
            'pure': card_number
        }

        if check_validation:
            result['is_valid'] = validate(card_number)

        if detect_bank_name and (result.get('is_valid') or validate(card_number)):
            result['bank_data'] = bank_data(card_number)

        results.append(result)

    if filter_valid_card_numbers and check_validation:
        results = [res for res in results if res['is_valid']]

    return results



OPERATORS = {
    'ShatelMobile': 'شاتل موبایل',
    'MCI': 'همراه اول',
    'Irancell': 'ایرانسل',
    'Taliya': 'تالیا',
    'RightTel': 'رایتل',
}

IRANCELL_SAMPLE = {
    'province': [],
    'base': 'کشوری',
    'type': ['permanent', 'credit'],
    'operator': OPERATORS['Irancell'],
}

data = {
    '910': {
        'base': 'کشوری',
        'province': [],
        'type': ['permanent', 'credit'],
        'operator': OPERATORS['MCI'],
    },
    '914': {
        'province': ['آذربایجان شرقی', 'اردبیل', 'اصفهان'],
        'base': 'آذربایجان غربی',
        'type': ['permanent', 'credit'],
        'operator': OPERATORS['MCI'],
    },
    '911': {
        'province': ['گلستان', 'گیلان'],
        'base': 'مازندران',
        'type': ['permanent', 'credit'],
        'operator': OPERATORS['MCI'],
    },
    '912': {
        'province': ['البرز', 'زنجان', 'سمنان', 'قزوین', 'قم', 'برخی از شهرستان های استان مرکزی'],
        'base': 'تهران',
        'type': ['permanent'],
        'operator': OPERATORS['MCI'],
    },
    '913': {
        'province': ['یزد', 'چهارمحال و بختیاری', 'کرمان'],
        'base': 'اصفهان',
        'type': ['permanent', 'credit'],
        'operator': OPERATORS['MCI'],
    },
    '915': {
        'province': ['خراسان شمالی', 'خراسان جنوبی', 'سیستان و بلوچستان'],
        'base': 'خراسان رضوی',
        'type': ['permanent', 'credit'],
        'operator': OPERATORS['MCI'],
    },
    '916': {
        'province': ['لرستان', 'فارس', 'اصفهان'],
        'base': 'خوزستان',
        'type': ['permanent', 'credit'],
        'operator': OPERATORS['MCI'],
    },
    '917': {
        'province': ['بوشهر', 'کهگیلویه و بویر احمد', 'هرمزگان'],
        'base': 'فارس',
        'type': ['permanent', 'credit'],
        'operator': OPERATORS['MCI'],
    },
    '918': {
        'province': ['کردستان', 'ایلام', 'همدان'],
        'base': 'کرمانشاه',
        'type': ['permanent', 'credit'],
        'operator': OPERATORS['MCI'],
    },
    '919': {
        'province': ['البرز', 'سمنان', 'قم', 'قزوین', 'زنجان'],
        'base': 'تهران',
        'type': ['credit'],
        'operator': OPERATORS['MCI'],
    },
    '990': {
        'province': [],
        'base': 'کشوری',
        'type': ['credit'],
        'operator': OPERATORS['MCI'],
    },
    '991': {
        'province': [],
        'base': 'کشوری',
        'type': ['permanent', 'credit'],
        'operator': OPERATORS['MCI'],
    },
    '992': {
        'province': [],
        'base': 'کشوری',
        'type': ['credit'],
        'operator': OPERATORS['MCI'],
    },
    '993': {
        'province': [],
        'base': 'کشوری',
        'type': ['credit'],
        'operator': OPERATORS['MCI'],
    },
    '994': {
        'province': [],
        'base': 'کشوری',
        'type': ['credit'],
        'operator': OPERATORS['MCI'],
    },
    '932': {
        'province': [],
        'base': 'کشوری',
        'type': ['credit'],
        'operator': OPERATORS['Taliya'],
    },
    '920': {
        'province': [],
        'base': 'کشوری',
        'type': ['permanent'],
        'operator': OPERATORS['RightTel'],
    },
    '921': {
        'province': [],
        'base': 'کشوری',
        'type': ['credit'],
        'operator': OPERATORS['RightTel'],
    },
    '922': {
        'province': [],
        'base': 'کشوری',
        'type': ['credit'],
        'operator': OPERATORS['RightTel'],
    },
    '930': IRANCELL_SAMPLE,
    '933': IRANCELL_SAMPLE,
    '935': IRANCELL_SAMPLE,
    '936': IRANCELL_SAMPLE,
    '937': IRANCELL_SAMPLE,
    '938': IRANCELL_SAMPLE,
    '939': IRANCELL_SAMPLE,
    '901': IRANCELL_SAMPLE,
    '902': IRANCELL_SAMPLE,
    '903': IRANCELL_SAMPLE,
    '905': IRANCELL_SAMPLE,
    '904': {
        'province': [],
        'base': 'کشوری',
        'model': 'سیم‌کارت کودک',
        'type': ['credit'],
        'operator': OPERATORS['Irancell'],
    },
    '941': {
        'province': [],
        'base': 'کشوری',
        'model': 'TD-LTE',
        'type': ['credit'],
        'operator': OPERATORS['Irancell'],
    },
    '998': {
        'province': [],
        'base': 'کشوری',
        'type': ['credit'],
        'operator': OPERATORS['ShatelMobile'],
    },
}



import setuptools
from importlib import import_module

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name='persian_tools',
    packages=setuptools.find_packages(),
    version=import_module('persian_tools').__version__,
    license='MIT',
    description='An anthology of a variety of tools for the Persian language in Python',
    long_description=long_description,
    long_description_content_type="text/markdown",
    author='Ali Madihi (mrunderline)',
    author_email='alimadihib@gmail.com',
    url='https://github.com/persian-tools/py-persian-tools',
    download_url='https://github.com/persian-tools/py-persian-tools/archive/v0.0.9.zip',
    keywords=['digits', 'commas', 'iranian bank', 'card number', 'national id', 'national code', 'ordinal suffix'],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Plugins',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Topic :: Utilities'
    ],
)



from persian_tools import plate
from persian_tools.plate.exceptions import InvalidPlateLength
import pytest


def test_plate_type():
    assert plate.get_info('12ب14547').get('type') == plate.TYPE_CAR
    assert plate.get_info('12ب145478').get('type') == plate.TYPE_MOTORCYCLE

    with pytest.raises(InvalidPlateLength, match='.*digits long, must be 7 or 8'):
        plate.get_info('12ب1454789').get('type')


def test_car_plate_info():
    actual = plate.get_info('12ب14547')
    expected = {
        'type': plate.TYPE_CAR,
        'template': '12ب145ایران47',
        'province': 'مرکزی',
        'category': 'شخصی'
    }
    assert all(v == actual[k] for k, v in expected.items())
    assert len(expected) == len(actual)

    actual = plate.get_info('12ب14547')
    assert all(v == actual[k] for k, v in expected.items())
    assert len(expected) == len(actual)

    actual = plate.get_info('1214501')
    expected = {
        'type': plate.TYPE_CAR,
        'template': '12None145ایران1',
        'province': None,
        'category': None
    }
    assert all(v == actual[k] for k, v in expected.items())
    assert len(expected) == len(actual)


def test_motorcycle_plate_info():
    actual = plate.get_info('12145478')
    expected = {
        'type': plate.TYPE_MOTORCYCLE,
        'template': '121-45478',
        'province': 'مرکز تهران',
        'category': None
    }
    assert all(v == actual[k] for k, v in expected.items())
    assert len(expected) == len(actual)

    actual = plate.get_info('10045118')
    expected = {
        'type': plate.TYPE_MOTORCYCLE,
        'template': '100-45118',
        'province': None,
        'category': None
    }
    assert all(v == actual[k] for k, v in expected.items())
    assert len(expected) == len(actual)


def test_is_valid_function():
    assert plate.is_valid('12ب14547')  # valid car plate
    assert plate.is_valid('12145478')  # valid motorcycle plate
    assert plate.is_valid('12ب145774347') is False  # long plate number, for car plate
    assert plate.is_valid('121454789') is False  # long plate number, for motorcycle plate
    assert plate.is_valid('1214501') is False  # fake province code (01), for car plate
    assert plate.is_valid('10045118') is False  # fake province code (100), for motorcycle plate
    assert plate.is_valid('12g45147') is False  # invalid car plate character
    assert plate.is_valid('12الف45150') is False  # fake province code (50), for car plate
    assert plate.is_valid('10045678') is False  # fake province code (100), for motorcycle plate
    assert plate.is_valid('10045678') is False  # fake province code (100), for motorcycle plate

    assert plate.is_valid('12345678')
    assert plate.is_valid('12345670')
    assert plate.is_valid('1230567') is False
    assert plate.is_valid('12305678') is False
    assert plate.is_valid('1ی23456') is False
    assert plate.is_valid('1234f560') is False
    assert plate.is_valid('123450d0') is False


def test_normalize():
    actual = plate.normalize('1234567الف')
    expected = {
        'numbers': '1234567',
        'char': 'الف',
    }
    assert all(v == actual[k] for k, v in expected.items())
    assert len(expected) == len(actual)

    actual = plate.normalize('1234567')
    expected = {
        'numbers': '1234567',
        'char': None,
    }
    assert all(v == actual[k] for k, v in expected.items())
    assert len(expected) == len(actual)



from persian_tools import bill
from persian_tools.bill.exceptions import NotEnoughArguments
import pytest


def test_bill_amount():
    assert bill.get_detail(bill_id=1117753200140, payment_id=12070160, currency=bill.CURRENCY_RIAL).get(
        'amount') == 120000
    assert bill.get_detail(bill_id=1117753200140, payment_id=12070160).get('amount') == 12000
    assert bill.get_detail(bill_id=1177809000142, payment_id=570108, currency=bill.CURRENCY_RIAL).get('amount') == 5000
    assert bill.get_detail(bill_id=1177809000142, payment_id=570108).get('amount') == 500
    assert bill.get_detail(bill_id=7748317800142, payment_id=1770160, currency=bill.CURRENCY_RIAL).get(
        'amount') == 17000
    assert bill.get_detail(bill_id=7748317800142, payment_id=1770160).get('amount') == 1700


def test_bill_type():
    assert bill.get_detail(bill_id=7748317800142, payment_id=1770160, currency=bill.CURRENCY_RIAL).get(
        'type') == 'تلفن ثابت'
    assert bill.get_detail(bill_id=9174639504124, payment_id=12908197).get('type') == 'برق'
    assert bill.get_detail(bill_id=2050327604613, payment_id=1070189).get('type') == 'آب'
    assert bill.get_detail(bill_id=9100074409151, payment_id=12908190).get('type') == 'تلفن همراه'


def test_bill_id_validation():
    assert bill.get_detail(bill_id=7748317800142, payment_id=1770160).get('is_valid_bill_id') is True
    assert bill.get_detail(bill_id=9174639504124, payment_id=12908197).get('is_valid_bill_id') is True
    assert bill.get_detail(bill_id=2050327604613, payment_id=1070189).get('is_valid_bill_id') is True
    assert bill.get_detail(bill_id=2234322344613, payment_id=1070189).get('is_valid_bill_id') is False
    assert bill.get_detail(bill_id=2234, payment_id=1070189).get('is_valid_bill_id') is False


def test_bill_payment_id_validation():
    assert bill.get_detail(bill_id=7748317800142, payment_id=1770160).get('is_valid_payment_id') is True
    assert bill.get_detail(bill_id=9174639504124, payment_id=12908197).get('is_valid_payment_id') is False
    assert bill.get_detail(bill_id=2050327604613, payment_id=1070189).get('is_valid_payment_id') is True
    assert bill.get_detail(bill_id=2234322344613, payment_id=1070189).get('is_valid_payment_id') is False
    assert bill.get_detail(bill_id=2234322344613, payment_id=1070).get('is_valid_payment_id') is False


def test_bill_validation():
    assert bill.get_detail(bill_id=7748317800142, payment_id=1770160).get('is_valid') is True
    assert bill.get_detail(bill_id=9174639504124, payment_id=12908197).get('is_valid') is False
    assert bill.get_detail(bill_id=2050327604613, payment_id=1070189).get('is_valid') is True
    assert bill.get_detail(bill_id=2234322344613, payment_id=1070189).get('is_valid') is False


def test_find_bills_barcode():
    assert bill.get_detail(bill_id=7748317800142, payment_id=1770160).get('barcode') == '77483178001420001770160'
    assert bill.get_detail(bill_id=9174639504124, payment_id=12908197).get('barcode') == '917463950412400012908197'
    assert bill.get_detail(bill_id=2050327604613, payment_id=1070189).get('barcode') == '20503276046130001070189'
    assert bill.get_detail(bill_id=2234322344613, payment_id=1070189).get('barcode') == '22343223446130001070189'


def test_find_bills_id():
    bill_detail = bill.get_detail(barcode='22343223446130001070189')
    assert bill_detail.get('bill_id') == 2234322344613
    assert bill_detail.get('payment_id') == 1070189


def test_not_enough_arguments():
    with pytest.raises(NotEnoughArguments,
                       match='Arguments are not enough; we need `bill_id` and `payment_id` or at least, bill `barcode`'):
        bill.get_detail()
        bill.get_detail(bill_id=7748317800142)



class InvalidPlateLength(Exception):
    def __init__(self, plate):
        self.card_number = plate
        self.message = plate + ' digits long, must be 7 or 8'
        super().__init__(self.message)
